@@ -3,7 +3,8 @@
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; version 2 of the License.
+ * the Free Software Foundation; either version 2 of the License, or
+ * (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful, but
  * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
@@ -22,7 +23,8 @@ namespace OCC {
    @brief The AccountManager class
    @ingroup gui
 */
-class AccountManager : public QObject {
+class AccountManager : public QObject
+{
     Q_OBJECT
 public:
     static AccountManager *instance();
@@ -35,7 +37,9 @@ public:
 
     /**
      * Creates account objects from a given settings file.
-     * return true if the account was restored
+     *
+     * Returns false if there was an error reading the settings,
+     * but note that settings not existing is not an error.
      */
     bool restore();
 
@@ -59,7 +63,7 @@ public:
     /**
      * Return the account state pointer for an account identified by its display name
      */
-    AccountStatePtr account(const QString& name);
+    AccountStatePtr account(const QString &name);
 
     /**
      * Delete the AccountState
@@ -75,23 +79,23 @@ public:
 
 private:
     // saving and loading Account to settings
-    void saveAccountHelper(Account* account, QSettings& settings, bool saveCredentials = true);
-    AccountPtr loadAccountHelper(QSettings& settings);
+    void saveAccountHelper(Account *account, QSettings &settings, bool saveCredentials = true);
+    AccountPtr loadAccountHelper(QSettings &settings);
 
     bool restoreFromLegacySettings();
 
-    bool isAccountIdAvailable(const QString& id) const;
+    bool isAccountIdAvailable(const QString &id) const;
     QString generateFreeAccountId() const;
 
     // Adds an account to the tracked list, emitting accountAdded()
-    void addAccountState(AccountState* accountState);
+    void addAccountState(AccountState *accountState);
 
 public slots:
     /// Saves account data, not including the credentials
-    void saveAccount(Account* a);
+    void saveAccount(Account *a);
 
     /// Saves account state data, not including the account
-    void saveAccountState(AccountState* a);
+    void saveAccountState(AccountState *a);
 
 
 Q_SIGNALS:
@@ -102,5 +106,4 @@ private:
     AccountManager() {}
     QList<AccountStatePtr> _accounts;
 };
-
 }
