@@ -13,15 +13,12 @@
  */
 
 #include <QtGui>
-#if QT_VERSION >= QT_VERSION_CHECK(5, 0, 0)
 #include <QtWidgets>
-#endif
 
 #include "activitylistmodel.h"
 #include "activitywidget.h"
 #include "syncresult.h"
 #include "logger.h"
-#include "utility.h"
 #include "theme.h"
 #include "folderman.h"
 #include "syncfileitem.h"
@@ -33,6 +30,7 @@
 #include "accountmanager.h"
 #include "activityitemdelegate.h"
 #include "protocolwidget.h"
+#include "issueswidget.h"
 #include "QProgressIndicator.h"
 #include "notificationwidget.h"
 #include "notificationconfirmjob.h"
@@ -50,16 +48,14 @@
 
 namespace OCC {
 
-/* ==================================================================== */
-
-ActivityWidget::ActivityWidget(QWidget *parent) :
-    QWidget(parent),
-    _ui(new Ui::ActivityWidget),
-    _notificationRequestsRunning(0)
+ActivityWidget::ActivityWidget(QWidget *parent)
+    : QWidget(parent)
+    , _ui(new Ui::ActivityWidget)
+    , _notificationRequestsRunning(0)
 {
     _ui->setupUi(this);
 
-    // Adjust copyToClipboard() when making changes here!
+// Adjust copyToClipboard() when making changes here!
 #if defined(Q_OS_MAC)
     _ui->_activityList->setMinimumWidth(400);
 #endif
@@ -85,19 +81,18 @@ ActivityWidget::ActivityWidget(QWidget *parent) :
 
     showLabels();
 
-    connect(_model, SIGNAL(activityJobStatusCode(AccountState*,int)),
-            this, SLOT(slotAccountActivityStatus(AccountState*,int)));
+    connect(_model, &ActivityListModel::activityJobStatusCode,
+        this, &ActivityWidget::slotAccountActivityStatus);
 
     _copyBtn = _ui->_dialogButtonBox->addButton(tr("Copy"), QDialogButtonBox::ActionRole);
-    _copyBtn->setToolTip( tr("Copy the activity list to the clipboard."));
-    connect(_copyBtn, SIGNAL(clicked()), SIGNAL(copyToClipboard()));
+    _copyBtn->setToolTip(tr("Copy the activity list to the clipboard."));
+    connect(_copyBtn, &QAbstractButton::clicked, this, &ActivityWidget::copyToClipboard);
 
-    connect(_model, SIGNAL(rowsInserted(QModelIndex,int,int)), SIGNAL(rowsInserted()));
+    connect(_model, &QAbstractItemModel::rowsInserted, this, &ActivityWidget::rowsInserted);
 
-    connect( _ui->_activityList, SIGNAL(activated(QModelIndex)), this,
-             SLOT(slotOpenFile(QModelIndex)));
+    connect(_ui->_activityList, &QListView::activated, this, &ActivityWidget::slotOpenFile);
 
-    connect( &_removeTimer, SIGNAL(timeout()), this, SLOT(slotCheckToCleanWidgets()) );
+    connect(&_removeTimer, &QTimer::timeout, this, &ActivityWidget::slotCheckToCleanWidgets);
     _removeTimer.setInterval(1000);
 }
 
@@ -115,18 +110,18 @@ void ActivityWidget::slotRefreshNotifications(AccountState *ptr)
 {
     // start a server notification handler if no notification requests
     // are running
-    if( _notificationRequestsRunning == 0 ) {
+    if (_notificationRequestsRunning == 0) {
         ServerNotificationHandler *snh = new ServerNotificationHandler;
-        connect(snh, SIGNAL(newNotificationList(ActivityList)), this,
-                SLOT(slotBuildNotificationDisplay(ActivityList)));
+        connect(snh, &ServerNotificationHandler::newNotificationList,
+            this, &ActivityWidget::slotBuildNotificationDisplay);
 
         snh->slotFetchNotifications(ptr);
     } else {
-        qDebug() << Q_FUNC_INFO << "========> notification request counter not zero.";
+        qCWarning(lcActivity) << "Notification request counter not zero.";
     }
 }
 
-void ActivityWidget::slotRemoveAccount( AccountState *ptr )
+void ActivityWidget::slotRemoveAccount(AccountState *ptr)
 {
     _model->slotRemoveAccount(ptr);
 }
@@ -141,8 +136,8 @@ void ActivityWidget::showLabels()
 
     t.clear();
     QSetIterator<QString> i(_accountsWithoutActivities);
-    while (i.hasNext() ) {
-        t.append( tr("<br/>Account %1 does not have activities enabled.").arg(i.next()));
+    while (i.hasNext()) {
+        t.append(tr("<br/>Account %1 does not have activities enabled.").arg(i.next()));
     }
     _ui->_bottomLabel->setTextFormat(Qt::RichText);
     _ui->_bottomLabel->setText(t);
@@ -150,10 +145,10 @@ void ActivityWidget::showLabels()
 
 void ActivityWidget::slotAccountActivityStatus(AccountState *ast, int statusCode)
 {
-    if( !(ast && ast->account()) ) {
+    if (!(ast && ast->account())) {
         return;
     }
-    if( statusCode == 999 ) {
+    if (statusCode == 999) {
         _accountsWithoutActivities.insert(ast->account()->displayName());
     } else {
         _accountsWithoutActivities.remove(ast->account()->displayName());
@@ -173,40 +168,40 @@ QString ActivityWidget::timeString(QDateTime dt, QLocale::FormatType format) con
     return loc.toString(dt, dtFormat);
 }
 
-void ActivityWidget::storeActivityList( QTextStream& ts )
+void ActivityWidget::storeActivityList(QTextStream &ts)
 {
     ActivityList activities = _model->activityList();
 
-    foreach( Activity activity, activities ) {
+    foreach (Activity activity, activities) {
         ts << right
-              // account name
+           // account name
            << qSetFieldWidth(30)
            << activity._accName
-              // separator
+           // separator
            << qSetFieldWidth(0) << ","
 
-              // date and time
+           // date and time
            << qSetFieldWidth(34)
            << activity._dateTime.toString()
-              // separator
+           // separator
            << qSetFieldWidth(0) << ","
 
-              // file
+           // file
            << qSetFieldWidth(30)
            << activity._file
-              // separator
+           // separator
            << qSetFieldWidth(0) << ","
 
-              // subject
+           // subject
            << qSetFieldWidth(100)
            << activity._subject
-              // separator
+           // separator
            << qSetFieldWidth(0) << ","
 
-              // message (mostly empty)
+           // message (mostly empty)
            << qSetFieldWidth(55)
            << activity._message
-              //
+           //
            << qSetFieldWidth(0)
            << endl;
     }
@@ -216,22 +211,22 @@ void ActivityWidget::checkActivityTabVisibility()
 {
     int accountCount = AccountManager::instance()->accounts().count();
     bool hasAccountsWithActivity =
-            _accountsWithoutActivities.count() != accountCount;
+        _accountsWithoutActivities.count() != accountCount;
     bool hasNotifications = !_widgetForNotifId.isEmpty();
 
-    _ui->_headerLabel->setVisible( hasAccountsWithActivity );
-    _ui->_activityList->setVisible( hasAccountsWithActivity );
+    _ui->_headerLabel->setVisible(hasAccountsWithActivity);
+    _ui->_activityList->setVisible(hasAccountsWithActivity);
 
-    _ui->_notifyLabel->setVisible( hasNotifications );
-    _ui->_notifyScroll->setVisible( hasNotifications );
+    _ui->_notifyLabel->setVisible(hasNotifications);
+    _ui->_notifyScroll->setVisible(hasNotifications);
 
     emit hideActivityTab(!hasAccountsWithActivity && !hasNotifications);
 }
 
 void ActivityWidget::slotOpenFile(QModelIndex indx)
 {
-    qDebug() << Q_FUNC_INFO << indx.isValid() << indx.data(ActivityItemDelegate::PathRole).toString() << QFile::exists(indx.data(ActivityItemDelegate::PathRole).toString());
-    if( indx.isValid() ) {
+    qCDebug(lcActivity) << indx.isValid() << indx.data(ActivityItemDelegate::PathRole).toString() << QFile::exists(indx.data(ActivityItemDelegate::PathRole).toString());
+    if (indx.isValid()) {
         QString fullPath = indx.data(ActivityItemDelegate::PathRole).toString();
 
         if (QFile::exists(fullPath)) {
@@ -244,7 +239,7 @@ void ActivityWidget::slotOpenFile(QModelIndex indx)
 // All notifications in list are coming from the same account
 // but in the _widgetForNotifId hash widgets for all accounts are
 // collected.
-void ActivityWidget::slotBuildNotificationDisplay(const ActivityList& list)
+void ActivityWidget::slotBuildNotificationDisplay(const ActivityList &list)
 {
     QHash<QString, int> accNotified;
     QString listAccountName;
@@ -252,33 +247,31 @@ void ActivityWidget::slotBuildNotificationDisplay(const ActivityList& list)
     // Whether a new notification widget was added to the notificationLayout.
     bool newNotificationShown = false;
 
-    foreach( auto activity, list ) {
-        if( _blacklistedNotifications.contains(activity)) {
-            qDebug() << Q_FUNC_INFO << "Activity in blacklist, skip";
+    foreach (auto activity, list) {
+        if (_blacklistedNotifications.contains(activity)) {
+            qCInfo(lcActivity) << "Activity in blacklist, skip";
             continue;
         }
 
         NotificationWidget *widget = 0;
 
-        if( _widgetForNotifId.contains( activity.ident()) ) {
+        if (_widgetForNotifId.contains(activity.ident())) {
             widget = _widgetForNotifId[activity.ident()];
         } else {
             widget = new NotificationWidget(this);
-            connect(widget, SIGNAL(sendNotificationRequest(QString, QString, QByteArray)),
-                    this, SLOT(slotSendNotificationRequest(QString, QString, QByteArray)));
-            connect(widget, SIGNAL(requestCleanupAndBlacklist(Activity)),
-                    this, SLOT(slotRequestCleanupAndBlacklist(Activity)));
+            connect(widget, &NotificationWidget::sendNotificationRequest,
+                this, &ActivityWidget::slotSendNotificationRequest);
+            connect(widget, &NotificationWidget::requestCleanupAndBlacklist,
+                this, &ActivityWidget::slotRequestCleanupAndBlacklist);
 
             _notificationsLayout->addWidget(widget);
-            // _ui->_notifyScroll->setMinimumHeight( widget->height());
-#if QT_VERSION >= QT_VERSION_CHECK(5, 2, 0)
+// _ui->_notifyScroll->setMinimumHeight( widget->height());
             _ui->_notifyScroll->setSizeAdjustPolicy(QAbstractScrollArea::AdjustToContentsOnFirstShow);
-#endif
             _widgetForNotifId[activity.ident()] = widget;
             newNotificationShown = true;
         }
 
-        widget->setActivity( activity );
+        widget->setActivity(activity);
 
         // remember the list account name for the strayCat handling below.
         listAccountName = activity._accName;
@@ -290,21 +283,21 @@ void ActivityWidget::slotBuildNotificationDisplay(const ActivityList& list)
         // will repeat the gui log.
 
         // after one hour, clear the gui log notification store
-        if( _guiLogTimer.elapsed() > 60*60*1000 ) {
+        if (_guiLogTimer.elapsed() > 60 * 60 * 1000) {
             _guiLoggedNotifications.clear();
         }
-        if( !_guiLoggedNotifications.contains(activity._id)) {
+        if (!_guiLoggedNotifications.contains(activity._id)) {
             QString host = activity._accName;
             // store the name of the account that sends the notification to be
             // able to add it to the tray notification
             // remove the user name from the account as that is not accurate here.
             int indx = host.indexOf(QChar('@'));
-            if( indx>-1 ) {
-                host.remove(0, 1+indx);
+            if (indx > -1) {
+                host.remove(0, 1 + indx);
             }
-            if( !host.isEmpty() ) {
-                if( accNotified.contains(host)) {
-                    accNotified[host] = accNotified[host]+1;
+            if (!host.isEmpty()) {
+                if (accNotified.contains(host)) {
+                    accNotified[host] = accNotified[host] + 1;
                 } else {
                     accNotified[host] = 1;
                 }
@@ -315,31 +308,31 @@ void ActivityWidget::slotBuildNotificationDisplay(const ActivityList& list)
 
     // check if there are widgets that have no corresponding activity from
     // the server any more. Collect them in a list
-    QList< Activity::Identifier > strayCats;
-    foreach( auto id, _widgetForNotifId.keys() ) {
+    QList<Activity::Identifier> strayCats;
+    foreach (auto id, _widgetForNotifId.keys()) {
         NotificationWidget *widget = _widgetForNotifId[id];
 
         bool found = false;
         // do not mark widgets of other accounts to delete.
-        if( widget->activity()._accName != listAccountName ) {
+        if (widget->activity()._accName != listAccountName) {
             continue;
         }
 
-        foreach( auto activity, list ) {
-            if( activity.ident() == id ) {
+        foreach (auto activity, list) {
+            if (activity.ident() == id) {
                 // found an activity
                 found = true;
                 break;
             }
         }
-        if( ! found ) {
+        if (!found) {
             // the activity does not exist any more.
             strayCats.append(id);
         }
     }
 
     // .. and now delete all these stray cat widgets.
-    foreach( auto strayCatId, strayCats ) {
+    foreach (auto strayCatId, strayCats) {
         NotificationWidget *widgetToGo = _widgetForNotifId[strayCatId];
         scheduleWidgetToRemove(widgetToGo, 0);
     }
@@ -348,19 +341,18 @@ void ActivityWidget::slotBuildNotificationDisplay(const ActivityList& list)
 
     int newGuiLogCount = accNotified.count();
 
-    if( newGuiLogCount > 0 ) {
+    if (newGuiLogCount > 0) {
         // restart the gui log timer now that we show a notification
-        _guiLogTimer.restart();
+        _guiLogTimer.start();
 
         // Assemble a tray notification
-        QString msg = tr("You received %n new notification(s) from %2.", "", accNotified[accNotified.keys().at(0)]).
-                arg(accNotified.keys().at(0));
+        QString msg = tr("You received %n new notification(s) from %2.", "", accNotified[accNotified.keys().at(0)]).arg(accNotified.keys().at(0));
 
-        if( newGuiLogCount >= 2 ) {
+        if (newGuiLogCount >= 2) {
             QString acc1 = accNotified.keys().at(0);
             QString acc2 = accNotified.keys().at(1);
-            if( newGuiLogCount == 2 ) {
-                int notiCount = accNotified[ acc1 ] + accNotified[ acc2 ];
+            if (newGuiLogCount == 2) {
+                int notiCount = accNotified[acc1] + accNotified[acc2];
                 msg = tr("You received %n new notification(s) from %1 and %2.", "", notiCount).arg(acc1, acc2);
             } else {
                 msg = tr("You received new notifications from %1, %2 and other accounts.").arg(acc1, acc2);
@@ -368,7 +360,7 @@ void ActivityWidget::slotBuildNotificationDisplay(const ActivityList& list)
         }
 
         const QString log = tr("%1 Notifications - Action Required").arg(Theme::instance()->appNameGUI());
-        emit guiLog( log, msg);
+        emit guiLog(log, msg);
     }
 
     if (newNotificationShown) {
@@ -376,24 +368,27 @@ void ActivityWidget::slotBuildNotificationDisplay(const ActivityList& list)
     }
 }
 
-void ActivityWidget::slotSendNotificationRequest(const QString& accountName, const QString& link, const QByteArray& verb)
+void ActivityWidget::slotSendNotificationRequest(const QString &accountName, const QString &link, const QByteArray &verb)
 {
-    qDebug() << Q_FUNC_INFO << "Server Notification Request " << verb << link << "on account" << accountName;
-    NotificationWidget *theSender = qobject_cast<NotificationWidget*>(sender());
+    qCInfo(lcActivity) << "Server Notification Request " << verb << link << "on account" << accountName;
+    NotificationWidget *theSender = qobject_cast<NotificationWidget *>(sender());
 
-    const QStringList validVerbs = QStringList() << "GET" << "PUT" << "POST" << "DELETE";
+    const QStringList validVerbs = QStringList() << "GET"
+                                                 << "PUT"
+                                                 << "POST"
+                                                 << "DELETE";
 
-    if( validVerbs.contains(verb)) {
+    if (validVerbs.contains(verb)) {
         AccountStatePtr acc = AccountManager::instance()->account(accountName);
-        if( acc ) {
+        if (acc) {
             NotificationConfirmJob *job = new NotificationConfirmJob(acc->account());
             QUrl l(link);
             job->setLinkAndVerb(l, verb);
             job->setWidget(theSender);
-            connect( job, SIGNAL( networkError(QNetworkReply*)),
-                                  this, SLOT(slotNotifyNetworkError(QNetworkReply*)));
-            connect( job, SIGNAL( jobFinished(QString, int)),
-                     this, SLOT(slotNotifyServerFinished(QString, int)) );
+            connect(job, &AbstractNetworkJob::networkError,
+                this, &ActivityWidget::slotNotifyNetworkError);
+            connect(job, &NotificationConfirmJob::jobFinished,
+                this, &ActivityWidget::slotNotifyServerFinished);
             job->start();
 
             // count the number of running notification requests. If this member var
@@ -401,69 +396,64 @@ void ActivityWidget::slotSendNotificationRequest(const QString& accountName, con
             _notificationRequestsRunning++;
         }
     } else {
-        qDebug() << Q_FUNC_INFO << "Notification Links: Invalid verb:" << verb;
+        qCWarning(lcActivity) << "Notification Links: Invalid verb:" << verb;
     }
 }
 
-void ActivityWidget::endNotificationRequest( NotificationWidget *widget, int replyCode )
+void ActivityWidget::endNotificationRequest(NotificationWidget *widget, int replyCode)
 {
     _notificationRequestsRunning--;
-    if( widget ) {
+    if (widget) {
         widget->slotNotificationRequestFinished(replyCode);
     }
 }
 
-void ActivityWidget::slotNotifyNetworkError( QNetworkReply *reply)
+void ActivityWidget::slotNotifyNetworkError(QNetworkReply *reply)
 {
-    NotificationConfirmJob *job = qobject_cast<NotificationConfirmJob*>(sender());
-    if( !job ) {
+    NotificationConfirmJob *job = qobject_cast<NotificationConfirmJob *>(sender());
+    if (!job) {
         return;
     }
 
-    int resultCode =0;
-    if( reply ) {
-        resultCode = reply->attribute(QNetworkRequest::HttpStatusCodeAttribute).toInt();
-    }
+    int resultCode = reply->attribute(QNetworkRequest::HttpStatusCodeAttribute).toInt();
 
     endNotificationRequest(job->widget(), resultCode);
-    qDebug() << Q_FUNC_INFO << "Server notify job failed with code " << resultCode;
-
+    qCWarning(lcActivity) << "Server notify job failed with code " << resultCode;
 }
 
-void ActivityWidget::slotNotifyServerFinished( const QString& reply, int replyCode )
+void ActivityWidget::slotNotifyServerFinished(const QString &reply, int replyCode)
 {
-    NotificationConfirmJob *job = qobject_cast<NotificationConfirmJob*>(sender());
-    if( !job ) {
+    NotificationConfirmJob *job = qobject_cast<NotificationConfirmJob *>(sender());
+    if (!job) {
         return;
     }
 
     endNotificationRequest(job->widget(), replyCode);
-    // FIXME: remove the  widget after a couple of seconds
-    qDebug() << Q_FUNC_INFO << "Server Notification reply code"<< replyCode << reply;
+    qCInfo(lcActivity) << "Server Notification reply code" << replyCode << reply;
 
     // if the notification was successful start a timer that triggers
     // removal of the done widgets in a few seconds
     // Add 200 millisecs to the predefined value to make sure that the timer in
     // widget's method readyToClose() has elapsed.
-    if( replyCode == OCS_SUCCESS_STATUS_CODE ) {
-        scheduleWidgetToRemove( job->widget() );
+    if (replyCode == OCS_SUCCESS_STATUS_CODE || replyCode == OCS_SUCCESS_STATUS_CODE_V2) {
+        scheduleWidgetToRemove(job->widget());
     }
 }
 
 // blacklist the activity coming in here.
-void ActivityWidget::slotRequestCleanupAndBlacklist(const Activity& blacklistActivity)
+void ActivityWidget::slotRequestCleanupAndBlacklist(const Activity &blacklistActivity)
 {
-    if ( ! _blacklistedNotifications.contains(blacklistActivity) ) {
+    if (!_blacklistedNotifications.contains(blacklistActivity)) {
         _blacklistedNotifications.append(blacklistActivity);
     }
 
-    NotificationWidget *widget = _widgetForNotifId[ blacklistActivity.ident() ];
+    NotificationWidget *widget = _widgetForNotifId[blacklistActivity.ident()];
     scheduleWidgetToRemove(widget);
 }
 
 void ActivityWidget::scheduleWidgetToRemove(NotificationWidget *widget, int milliseconds)
 {
-    if( !widget ) {
+    if (!widget) {
         return;
     }
     // in five seconds from now, remove the widget.
@@ -472,7 +462,7 @@ void ActivityWidget::scheduleWidgetToRemove(NotificationWidget *widget, int mill
     if (!it.isValid() || it > removeTime) {
         it = removeTime;
     }
-    if( !_removeTimer.isActive() ) {
+    if (!_removeTimer.isActive()) {
         _removeTimer.start();
     }
 }
@@ -487,7 +477,7 @@ void ActivityWidget::slotCheckToCleanWidgets()
         QDateTime t = it.value();
         NotificationWidget *widget = it.key();
 
-        if( currentTime > t ) {
+        if (currentTime > t) {
             // found one to remove!
             Activity::Identifier id = widget->activity().ident();
             _widgetForNotifId.remove(id);
@@ -498,12 +488,12 @@ void ActivityWidget::slotCheckToCleanWidgets()
         }
     }
 
-    if( _widgetsToRemove.isEmpty() ) {
+    if (_widgetsToRemove.isEmpty()) {
         _removeTimer.stop();
     }
 
     // check to see if the whole notification pane should be hidden
-    if( _widgetForNotifId.isEmpty() ) {
+    if (_widgetForNotifId.isEmpty()) {
         _ui->_notifyLabel->setHidden(true);
         _ui->_notifyScroll->setHidden(true);
     }
@@ -513,7 +503,7 @@ void ActivityWidget::slotCheckToCleanWidgets()
 /* ==================================================================== */
 
 ActivitySettings::ActivitySettings(QWidget *parent)
-    :QWidget(parent)
+    : QWidget(parent)
 {
     QHBoxLayout *hbox = new QHBoxLayout(this);
     setLayout(hbox);
@@ -522,70 +512,64 @@ ActivitySettings::ActivitySettings(QWidget *parent)
     _tab = new QTabWidget(this);
     hbox->addWidget(_tab);
     _activityWidget = new ActivityWidget(this);
-    _activityTabId = _tab->insertTab(0, _activityWidget, Theme::instance()->applicationIcon(), tr("Server Activity"));
-    connect(_activityWidget, SIGNAL(copyToClipboard()), this, SLOT(slotCopyToClipboard()));
-    connect(_activityWidget, SIGNAL(hideActivityTab(bool)), this, SLOT(setActivityTabHidden(bool)));
-    connect(_activityWidget, SIGNAL(guiLog(QString,QString)), this, SIGNAL(guiLog(QString,QString)));
-    connect(_activityWidget, SIGNAL(newNotification()), SLOT(slotShowActivityTab()));
+    _activityTabId = _tab->addTab(_activityWidget, Theme::instance()->applicationIcon(), tr("Server Activity"));
+    connect(_activityWidget, &ActivityWidget::copyToClipboard, this, &ActivitySettings::slotCopyToClipboard);
+    connect(_activityWidget, &ActivityWidget::hideActivityTab, this, &ActivitySettings::setActivityTabHidden);
+    connect(_activityWidget, &ActivityWidget::guiLog, this, &ActivitySettings::guiLog);
+    connect(_activityWidget, &ActivityWidget::newNotification, this, &ActivitySettings::slotShowActivityTab);
 
     _protocolWidget = new ProtocolWidget(this);
-    _tab->insertTab(1, _protocolWidget, Theme::instance()->syncStateIcon(SyncResult::Success), tr("Sync Protocol"));
-    connect(_protocolWidget, SIGNAL(copyToClipboard()), this, SLOT(slotCopyToClipboard()));
-    connect(_protocolWidget, SIGNAL(issueItemCountUpdated(int)),
-            this, SLOT(slotShowIssueItemCount(int)));
+    _protocolTabId = _tab->addTab(_protocolWidget, Theme::instance()->syncStateIcon(SyncResult::Success), tr("Sync Protocol"));
+    connect(_protocolWidget, &ProtocolWidget::copyToClipboard, this, &ActivitySettings::slotCopyToClipboard);
 
-    // Add the not-synced list into the tab
-    QWidget *w = new QWidget;
-    QVBoxLayout *vbox2 = new QVBoxLayout(w);
-    vbox2->addWidget(new QLabel(tr("List of ignored or erroneous files"), this));
-    vbox2->addWidget(_protocolWidget->issueWidget());
-    QDialogButtonBox *dlgButtonBox = new QDialogButtonBox(this);
-    vbox2->addWidget(dlgButtonBox);
-    QPushButton *_copyBtn = dlgButtonBox->addButton(tr("Copy"), QDialogButtonBox::ActionRole);
-    _copyBtn->setToolTip( tr("Copy the activity list to the clipboard."));
-    _copyBtn->setEnabled(true);
-    connect(_copyBtn, SIGNAL(clicked()), this, SLOT(slotCopyToClipboard()));
-
-    w->setLayout(vbox2);
-    _syncIssueTabId = _tab->insertTab(2, w, Theme::instance()->syncStateIcon(SyncResult::Problem), QString());
+    _issuesWidget = new IssuesWidget(this);
+    _syncIssueTabId = _tab->addTab(_issuesWidget, Theme::instance()->syncStateIcon(SyncResult::Problem), QString());
     slotShowIssueItemCount(0); // to display the label.
+    connect(_issuesWidget, &IssuesWidget::issueCountUpdated,
+        this, &ActivitySettings::slotShowIssueItemCount);
+    connect(_issuesWidget, &IssuesWidget::copyToClipboard,
+        this, &ActivitySettings::slotCopyToClipboard);
 
     // Add a progress indicator to spin if the acitivity list is updated.
     _progressIndicator = new QProgressIndicator(this);
     _tab->setCornerWidget(_progressIndicator);
 
-    connect(&_notificationCheckTimer, SIGNAL(timeout()),
-            this, SLOT(slotRegularNotificationCheck()));
+    connect(&_notificationCheckTimer, &QTimer::timeout,
+        this, &ActivitySettings::slotRegularNotificationCheck);
 
     // connect a model signal to stop the animation.
-    connect(_activityWidget, SIGNAL(rowsInserted()), _progressIndicator, SLOT(stopAnimation()));
+    connect(_activityWidget, &ActivityWidget::rowsInserted, _progressIndicator, &QProgressIndicator::stopAnimation);
 
     // We want the protocol be the default
     _tab->setCurrentIndex(1);
 }
 
-void ActivitySettings::setNotificationRefreshInterval( quint64 interval )
+void ActivitySettings::setNotificationRefreshInterval(quint64 interval)
 {
-    qDebug() << "Starting Notification refresh timer with " << interval/1000 << " sec interval";
+    qCDebug(lcActivity) << "Starting Notification refresh timer with " << interval / 1000 << " sec interval";
     _notificationCheckTimer.start(interval);
 }
 
 void ActivitySettings::setActivityTabHidden(bool hidden)
 {
-    if( hidden && _activityTabId > -1 ) {
+    if (hidden && _activityTabId > -1) {
         _tab->removeTab(_activityTabId);
         _activityTabId = -1;
+        _protocolTabId -= 1;
+        _syncIssueTabId -= 1;
     }
 
-    if( !hidden && _activityTabId == -1 ) {
+    if (!hidden && _activityTabId == -1) {
         _activityTabId = _tab->insertTab(0, _activityWidget, Theme::instance()->applicationIcon(), tr("Server Activity"));
+        _protocolTabId += 1;
+        _syncIssueTabId += 1;
     }
 }
 
 void ActivitySettings::slotShowIssueItemCount(int cnt)
 {
     QString cntText = tr("Not Synced");
-    if( cnt ) {
+    if (cnt) {
         //: %1 is the number of not synced files.
         cntText = tr("Not Synced (%1)").arg(cnt);
     }
@@ -599,6 +583,15 @@ void ActivitySettings::slotShowActivityTab()
     }
 }
 
+void ActivitySettings::slotShowIssuesTab(const QString &folderAlias)
+{
+    if (_syncIssueTabId == -1)
+        return;
+    _tab->setCurrentIndex(_syncIssueTabId);
+
+    _issuesWidget->showFolderErrors(folderAlias);
+}
+
 void ActivitySettings::slotCopyToClipboard()
 {
     QString text;
@@ -607,46 +600,46 @@ void ActivitySettings::slotCopyToClipboard()
     int idx = _tab->currentIndex();
     QString message;
 
-    if( idx == 0 ) {
+    if (idx == _activityTabId) {
         // the activity widget
         _activityWidget->storeActivityList(ts);
         message = tr("The server activity list has been copied to the clipboard.");
-    } else if(idx == 1 ) {
+    } else if (idx == _protocolTabId) {
         // the protocol widget
         _protocolWidget->storeSyncActivity(ts);
         message = tr("The sync activity list has been copied to the clipboard.");
-    } else if(idx == 2 ) {
+    } else if (idx == _syncIssueTabId) {
         // issues Widget
         message = tr("The list of unsynced items has been copied to the clipboard.");
-       _protocolWidget->storeSyncIssues(ts);
+        _issuesWidget->storeSyncIssues(ts);
     }
 
     QApplication::clipboard()->setText(text);
     emit guiLog(tr("Copied to clipboard"), message);
 }
 
-void ActivitySettings::slotRemoveAccount( AccountState *ptr )
+void ActivitySettings::slotRemoveAccount(AccountState *ptr)
 {
     _activityWidget->slotRemoveAccount(ptr);
 }
 
-void ActivitySettings::slotRefresh( AccountState* ptr )
+void ActivitySettings::slotRefresh(AccountState *ptr)
 {
     // QElapsedTimer isn't actually constructed as invalid.
-    if ( !_timeSinceLastCheck.contains(ptr) ) {
+    if (!_timeSinceLastCheck.contains(ptr)) {
         _timeSinceLastCheck[ptr].invalidate();
     }
-    QElapsedTimer & timer = _timeSinceLastCheck[ptr];
+    QElapsedTimer &timer = _timeSinceLastCheck[ptr];
 
     // Fetch Activities only if visible and if last check is longer than 15 secs ago
-    if( timer.isValid() && timer.elapsed() < NOTIFICATION_REQUEST_FREE_PERIOD ) {
-        qDebug() << Q_FUNC_INFO << "do not check as last check is only secs ago: " << timer.elapsed() / 1000;
+    if (timer.isValid() && timer.elapsed() < NOTIFICATION_REQUEST_FREE_PERIOD) {
+        qCDebug(lcActivity) << "Do not check as last check is only secs ago: " << timer.elapsed() / 1000;
         return;
     }
-    if( ptr && ptr->isConnected() ) {
-        if( isVisible() || !timer.isValid() ) {
+    if (ptr && ptr->isConnected()) {
+        if (isVisible() || !timer.isValid()) {
             _progressIndicator->startAnimation();
-            _activityWidget->slotRefreshActivities( ptr);
+            _activityWidget->slotRefreshActivities(ptr);
         }
         _activityWidget->slotRefreshNotifications(ptr);
         timer.start();
@@ -661,7 +654,7 @@ void ActivitySettings::slotRegularNotificationCheck()
     }
 }
 
-bool ActivitySettings::event(QEvent* e)
+bool ActivitySettings::event(QEvent *e)
 {
     if (e->type() == QEvent::Show) {
         AccountManager *am = AccountManager::instance();
@@ -674,8 +667,5 @@ bool ActivitySettings::event(QEvent* e)
 
 ActivitySettings::~ActivitySettings()
 {
-
 }
-
-
 }
