@@ -3,7 +3,8 @@
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; version 2 of the License.
+ * the Free Software Foundation; either version 2 of the License, or
+ * (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful, but
  * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
@@ -34,12 +35,13 @@ namespace OCC {
 class Account;
 class AccountStatusPtr;
 class ProtocolWidget;
+class IssuesWidget;
 class JsonApiJob;
 class NotificationWidget;
 class ActivityListModel;
 
 namespace Ui {
-  class ActivityWidget;
+    class ActivityWidget;
 }
 class Application;
 
@@ -70,25 +72,25 @@ public:
 
 public slots:
     void slotOpenFile(QModelIndex indx);
-    void slotRefreshActivities(AccountState* ptr);
+    void slotRefreshActivities(AccountState *ptr);
     void slotRefreshNotifications(AccountState *ptr);
-    void slotRemoveAccount( AccountState *ptr );
+    void slotRemoveAccount(AccountState *ptr);
     void slotAccountActivityStatus(AccountState *ast, int statusCode);
-    void slotRequestCleanupAndBlacklist(const Activity& blacklistActivity);
+    void slotRequestCleanupAndBlacklist(const Activity &blacklistActivity);
 
 signals:
-    void guiLog(const QString&, const QString&);
+    void guiLog(const QString &, const QString &);
     void copyToClipboard();
     void rowsInserted();
     void hideActivityTab(bool);
-    void newNotificationList(const ActivityList& list);
+    void newNotification();
 
 private slots:
-    void slotBuildNotificationDisplay(const ActivityList& list);
-    void slotSendNotificationRequest(const QString &accountName, const QString& link, const QByteArray &verb);
-    void slotNotifyNetworkError( QNetworkReply* );
-    void slotNotifyServerFinished( const QString& reply, int replyCode );
-    void endNotificationRequest(NotificationWidget *widget , int replyCode);
+    void slotBuildNotificationDisplay(const ActivityList &list);
+    void slotSendNotificationRequest(const QString &accountName, const QString &link, const QByteArray &verb);
+    void slotNotifyNetworkError(QNetworkReply *);
+    void slotNotifyServerFinished(const QString &reply, int replyCode);
+    void endNotificationRequest(NotificationWidget *widget, int replyCode);
     void scheduleWidgetToRemove(NotificationWidget *widget, int milliseconds = 4500);
     void slotCheckToCleanWidgets();
 
@@ -99,12 +101,12 @@ private:
     QPushButton *_copyBtn;
 
     QSet<QString> _accountsWithoutActivities;
-    QMap<Activity::Identifier, NotificationWidget*> _widgetForNotifId;
+    QMap<Activity::Identifier, NotificationWidget *> _widgetForNotifId;
     QElapsedTimer _guiLogTimer;
     QSet<int> _guiLoggedNotifications;
     ActivityList _blacklistedNotifications;
 
-    QHash<NotificationWidget*, QDateTime> _widgetsToRemove;
+    QHash<NotificationWidget *, QDateTime> _widgetsToRemove;
     QTimer _removeTimer;
 
     // number of currently running notification requests. If non zero,
@@ -113,7 +115,6 @@ private:
 
     ActivityListModel *_model;
     QVBoxLayout *_notificationsLayout;
-
 };
 
 
@@ -133,33 +134,37 @@ public:
     QSize sizeHint() const Q_DECL_OVERRIDE { return ownCloudGui::settingsDialogSize(); }
 
 public slots:
-    void slotRefresh( AccountState* ptr );
-    void slotRemoveAccount( AccountState *ptr );
+    void slotRefresh(AccountState *ptr);
+    void slotRemoveAccount(AccountState *ptr);
 
-    void setNotificationRefreshInterval( quint64 interval );
+    void setNotificationRefreshInterval(quint64 interval);
+
+    void slotShowIssuesTab(const QString &folderAlias);
 
 private slots:
     void slotCopyToClipboard();
     void setActivityTabHidden(bool hidden);
     void slotRegularNotificationCheck();
     void slotShowIssueItemCount(int cnt);
+    void slotShowActivityTab();
 
 signals:
-    void guiLog(const QString&, const QString&);
+    void guiLog(const QString &, const QString &);
 
 private:
-    bool event(QEvent* e) Q_DECL_OVERRIDE;
+    bool event(QEvent *e) Q_DECL_OVERRIDE;
 
     QTabWidget *_tab;
     int _activityTabId;
+    int _protocolTabId;
     int _syncIssueTabId;
 
     ActivityWidget *_activityWidget;
     ProtocolWidget *_protocolWidget;
+    IssuesWidget *_issuesWidget;
     QProgressIndicator *_progressIndicator;
-    QTimer         _notificationCheckTimer;
-    QHash<AccountState*, QElapsedTimer>  _timeSinceLastCheck;
+    QTimer _notificationCheckTimer;
+    QHash<AccountState *, QElapsedTimer> _timeSinceLastCheck;
 };
-
 }
 #endif // ActivityWIDGET_H
