@@ -24,7 +24,6 @@
 #include "folderwatcher.h"
 #include "syncfileitem.h"
 
-class QSignalMapper;
 class TestFolderMan;
 
 namespace OCC {
@@ -37,13 +36,33 @@ class LockWatcher;
 /**
  * @brief The FolderMan class
  * @ingroup gui
+ *
+ * The FolderMan knows about all loaded folders and is responsible for
+ * scheduling them when necessary.
+ *
+ * A folder is scheduled if:
+ * - The configured force-sync-interval has expired
+ *   (_timeScheduler and slotScheduleFolderByTime())
+ *
+ * - A folder watcher receives a notification about a file change
+ *   (_folderWatchers and Folder::slotWatchedPathChanged())
+ *
+ * - The folder etag on the server has changed
+ *   (_etagPollTimer)
+ *
+ * - The locks of a monitored file are released
+ *   (_lockWatcher and slotWatchedFileUnlocked())
+ *
+ * - There was a sync error or a follow-up sync is requested
+ *   (_timeScheduler and slotScheduleFolderByTime()
+ *    and Folder::slotSyncFinished())
  */
 class FolderMan : public QObject
 {
     Q_OBJECT
 public:
     ~FolderMan();
-    static FolderMan* instance();
+    static FolderMan *instance();
 
     int setupFolders();
     int setupFoldersMigration();
@@ -52,48 +71,51 @@ public:
 
     /** Adds a folder for an account, ensures the journal is gone and saves it in the settings.
       */
-    Folder* addFolder(AccountState* accountState, const FolderDefinition& folderDefinition);
+    Folder *addFolder(AccountState *accountState, const FolderDefinition &folderDefinition);
+
+    /** Removes a folder */
+    void removeFolder(Folder *);
 
     /** Returns the folder which the file or directory stored in path is in */
-    Folder* folderForPath(const QString& path);
+    Folder *folderForPath(const QString &path);
 
     /**
       * returns a list of local files that exist on the local harddisk for an
       * incoming relative server path. The method checks with all existing sync
       * folders.
       */
-    QStringList findFileInLocalFolders( const QString& relPath, const AccountPtr acc );
+    QStringList findFileInLocalFolders(const QString &relPath, const AccountPtr acc);
 
     /** Returns the folder by alias or NULL if no folder with the alias exists. */
-    Folder *folder( const QString& );
+    Folder *folder(const QString &);
 
     /**
      * Migrate accounts from owncloud < 2.0
      * Creates a folder for a specific configuration, identified by alias.
      */
-    Folder* setupFolderFromOldConfigFile(const QString &, AccountState *account );
+    Folder *setupFolderFromOldConfigFile(const QString &, AccountState *account);
 
     /**
-     * Ensures that a given directory does not contain a .csync_journal.
+     * Ensures that a given directory does not contain a sync journal file.
      *
      * @returns false if the journal could not be removed, true otherwise.
      */
-    static bool ensureJournalGone(const QString &path);
+    static bool ensureJournalGone(const QString &journalDbFile);
 
     /** Creates a new and empty local directory. */
-    bool startFromScratch( const QString& );
+    bool startFromScratch(const QString &);
 
-    QString statusToString(SyncResult, bool paused ) const;
+    QString statusToString(SyncResult::Status, bool paused) const;
 
-    static SyncResult accountStatus( const QList<Folder*> &folders );
+    static SyncResult accountStatus(const QList<Folder *> &folders);
 
-    void removeMonitorPath( const QString& alias, const QString& path );
-    void addMonitorPath( const QString& alias, const QString& path );
+    void removeMonitorPath(const QString &alias, const QString &path);
+    void addMonitorPath(const QString &alias, const QString &path);
 
     // Escaping of the alias which is used in QSettings AND the file
     // system, thus need to be escaped.
-    static QString escapeAlias( const QString& );
-    static QString unescapeAlias( const QString& );
+    static QString escapeAlias(const QString &);
+    static QString unescapeAlias(const QString &);
 
     SocketApi *socketApi();
 
@@ -101,11 +123,24 @@ public:
      * Check if @a path is a valid path for a new folder considering the already sync'ed items.
      * Make sure that this folder, or any subfolder is not sync'ed already.
      *
+     * Note that different accounts are allowed to sync to the same folder.
+     *
      * \a forNewDirectory is internal and is used for recursion.
      *
      * @returns an empty string if it is allowed, or an error if it is not allowed
      */
-    QString checkPathValidityForNewFolder(const QString &path, bool forNewDirectory = false);
+    QString checkPathValidityForNewFolder(const QString &path, const QUrl &serverUrl = QUrl(), bool forNewDirectory = false) const;
+
+    /**
+     * Attempts to find a non-existing, acceptable path for creating a new sync folder.
+     *
+     * Uses \a basePath as the baseline. It'll return this path if it's acceptable.
+     *
+     * Note that this can fail. If someone syncs ~ and \a basePath is ~/ownCloud, no
+     * subfolder of ~ would be a good candidate. When that happens \a basePath
+     * is returned.
+     */
+    QString findGoodPathForNewSyncFolder(const QString &basePath, const QUrl &serverUrl) const;
 
     /**
      * While ignoring hidden files can theoretically be switched per folder,
@@ -119,35 +154,33 @@ public:
     /**
      * Access to the current queue of scheduled folders.
      */
-    QQueue<Folder*> scheduleQueue() const;
+    QQueue<Folder *> scheduleQueue() const;
 
     /**
      * Access to the currently syncing folder.
      */
-    Folder* currentSyncFolder() const;
+    Folder *currentSyncFolder() const;
 
-signals:
-    /**
-      * signal to indicate a folder has changed its sync state.
-      *
-      * Attention: The folder may be zero. Do a general update of the state then.
-      */
-    void folderSyncStateChange(Folder*);
+    /** Removes all folders */
+    int unloadAndDeleteAllFolders();
 
     /**
-     * Indicates when the schedule queue changes.
+     * If enabled is set to false, no new folders will start to sync.
+     * The current one will finish.
      */
-    void scheduleQueueChanged();
+    void setSyncEnabled(bool);
 
-    void folderListChanged(const Folder::Map &);
+    /** Queues a folder for syncing. */
+    void scheduleFolder(Folder *);
 
-public slots:
-    void slotRemoveFolder( Folder* );
-    void slotFolderSyncPaused(Folder *, bool paused);
-    void slotFolderCanSyncChanged();
+    /** Puts a folder in the very front of the queue. */
+    void scheduleFolderNext(Folder *);
 
-    void slotFolderSyncStarted();
-    void slotFolderSyncFinished( const SyncResult& );
+    /** Queues all folders for syncing. */
+    void scheduleAllFolders();
+
+    void setDirtyProxy(bool value = true);
+    void setDirtyNetworkLimits();
 
     /**
      * Terminates the current folder sync.
@@ -156,24 +189,25 @@ public slots:
      */
     void terminateSyncProcess();
 
-    /* delete all folder objects */
-    int unloadAndDeleteAllFolders();
+signals:
+    /**
+      * signal to indicate a folder has changed its sync state.
+      *
+      * Attention: The folder may be zero. Do a general update of the state then.
+      */
+    void folderSyncStateChange(Folder *);
 
-    // if enabled is set to false, no new folders will start to sync.
-    // the current one will finish.
-    void setSyncEnabled( bool );
+    /**
+     * Indicates when the schedule queue changes.
+     */
+    void scheduleQueueChanged();
 
-    void slotScheduleAllFolders();
+    /**
+     * Emitted whenever the list of configured folders changes.
+     */
+    void folderListChanged(const Folder::Map &);
 
-    void setDirtyProxy(bool value = true);
-    void setDirtyNetworkLimits();
-
-    // slot to add a folder to the syncing queue
-    void slotScheduleSync(Folder*);
-    // slot to schedule an ETag job
-    void slotScheduleETagJob ( const QString &alias, RequestEtagJob *job);
-    void slotEtagJobDestroyed (QObject*);
-    void slotRunOneEtagJob();
+public slots:
 
     /**
      * Schedules folders of newly connected accounts, terminates and
@@ -190,68 +224,103 @@ public slots:
      * Triggers a sync run once the lock on the given file is removed.
      *
      * Automatically detemines the folder that's responsible for the file.
+     * See slotWatchedFileUnlocked().
      */
-    void slotSyncOnceFileUnlocks(const QString& path);
+    void slotSyncOnceFileUnlocks(const QString &path);
+
+    // slot to schedule an ETag job (from Folder only)
+    void slotScheduleETagJob(const QString &alias, RequestEtagJob *job);
 
 private slots:
+    void slotFolderSyncPaused(Folder *, bool paused);
+    void slotFolderCanSyncChanged();
+    void slotFolderSyncStarted();
+    void slotFolderSyncFinished(const SyncResult &);
+
+    void slotRunOneEtagJob();
+    void slotEtagJobDestroyed(QObject *);
+
     // slot to take the next folder from queue and start syncing.
     void slotStartScheduledFolderSync();
     void slotEtagPollTimerTimeout();
 
-    void slotRemoveFoldersForAccount(AccountState* accountState);
+    void slotRemoveFoldersForAccount(AccountState *accountState);
 
     // Wraps the Folder::syncStateChange() signal into the
     // FolderMan::folderSyncStateChange(Folder*) signal.
     void slotForwardFolderSyncStateChange();
 
-    void slotServerVersionChanged(Account* account);
+    void slotServerVersionChanged(Account *account);
 
     /**
-     * Schedules the folder for synchronization that contains
+     * A file whose locks were being monitored has become unlocked.
+     *
+     * This schedules the folder for synchronization that contains
      * the file with the given path.
      */
-    void slotScheduleFolderOwningFile(const QString& path);
+    void slotWatchedFileUnlocked(const QString &path);
+
+    /**
+     * Schedules folders whose time to sync has come.
+     *
+     * Either because a long time has passed since the last sync or
+     * because of previous failures.
+     */
+    void slotScheduleFolderByTime();
 
 private:
     /** Adds a new folder, does not add it to the account settings and
      *  does not set an account on the new folder.
       */
-    Folder* addFolderInternal(FolderDefinition folderDefinition, AccountState* accountState);
+    Folder *addFolderInternal(FolderDefinition folderDefinition,
+        AccountState *accountState);
 
     /* unloads a folder object, does not delete it */
-    void unloadFolder( Folder * );
+    void unloadFolder(Folder *);
 
     /** Will start a sync after a bit of delay. */
-    void startScheduledSyncSoon(qint64 msMinimumDelay = 0);
+    void startScheduledSyncSoon();
 
     // finds all folder configuration files
     // and create the folders
-    QString getBackupName( QString fullPathName ) const;
-    void registerFolderMonitor( Folder *folder );
+    QString getBackupName(QString fullPathName) const;
+    void registerFolderMonitor(Folder *folder);
 
     // restarts the application (Linux only)
     void restartApplication();
 
-    QSet<Folder*>  _disabledFolders;
-    Folder::Map    _folderMap;
-    QString        _folderConfigPath;
-    Folder        *_currentSyncFolder;
-    QPointer<Folder> _lastSyncFolder;
-    bool           _syncEnabled;
-    QTimer         _etagPollTimer;
-    QPointer<RequestEtagJob>        _currentEtagJob; // alias of Folder running the current RequestEtagJob
+    void setupFoldersHelper(QSettings &settings, AccountStatePtr account, bool backwardsCompatible);
 
-    QMap<QString, FolderWatcher*> _folderWatchers;
+    QSet<Folder *> _disabledFolders;
+    Folder::Map _folderMap;
+    QString _folderConfigPath;
+    Folder *_currentSyncFolder;
+    QPointer<Folder> _lastSyncFolder;
+    bool _syncEnabled;
+
+    /// Watching for file changes in folders
+    QMap<QString, FolderWatcher *> _folderWatchers;
+
+    /// Starts regular etag query jobs
+    QTimer _etagPollTimer;
+    /// The currently running etag query
+    QPointer<RequestEtagJob> _currentEtagJob;
+
+    /// Watches files that couldn't be synced due to locks
     QScopedPointer<LockWatcher> _lockWatcher;
+
+    /// Occasionally schedules folders
+    QTimer _timeScheduler;
+
+    /// Scheduled folders that should be synced as soon as possible
+    QQueue<Folder *> _scheduledFolders;
+
+    /// Picks the next scheduled folder and starts the sync
+    QTimer _startScheduledSyncTimer;
+
     QScopedPointer<SocketApi> _socketApi;
 
-    /** The aliases of folders that shall be synced. */
-    QQueue<Folder*> _scheduleQueue;
-
-    /** When the timer expires one of the scheduled syncs will be started. */
-    QTimer          _startScheduledSyncTimer;
-
-    bool            _appRestartRequired;
+    bool _appRestartRequired;
 
     static FolderMan *_instance;
     explicit FolderMan(QObject *parent = 0);
