@@ -3,7 +3,8 @@
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; version 2 of the License.
+ * the Free Software Foundation; either version 2 of the License, or
+ * (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful, but
  * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
@@ -18,7 +19,6 @@
 
 #include <QFileInfo>
 #include <QFlags>
-#include <QDebug>
 #include <QDir>
 #include <QMutexLocker>
 #include <QStringList>
@@ -37,9 +37,11 @@
 
 namespace OCC {
 
-FolderWatcher::FolderWatcher(const QString &root, Folder* folder)
-    : QObject(folder),
-      _folder(folder)
+Q_LOGGING_CATEGORY(lcFolderWatcher, "gui.folderwatcher", QtInfoMsg)
+
+FolderWatcher::FolderWatcher(const QString &root, Folder *folder)
+    : QObject(folder)
+    , _folder(folder)
 {
     _d.reset(new FolderWatcherPrivate(this, root));
 
@@ -47,32 +49,33 @@ FolderWatcher::FolderWatcher(const QString &root, Folder* folder)
 }
 
 FolderWatcher::~FolderWatcher()
-{ }
-
-bool FolderWatcher::pathIsIgnored( const QString& path )
 {
-    if( path.isEmpty() ) return true;
-    if( !_folder ) return false;
+}
+
+bool FolderWatcher::pathIsIgnored(const QString &path)
+{
+    if (path.isEmpty())
+        return true;
+    if (!_folder)
+        return false;
 
 #ifndef OWNCLOUD_TEST
     if (_folder->isFileExcludedAbsolute(path)) {
-        qDebug() << "* Ignoring file" << path;
+        qCDebug(lcFolderWatcher) << "* Ignoring file" << path;
         return true;
     }
 #endif
     return false;
 }
 
-void FolderWatcher::changeDetected( const QString& path )
+void FolderWatcher::changeDetected(const QString &path)
 {
     QStringList paths(path);
     changeDetected(paths);
 }
 
-void FolderWatcher::changeDetected( const QStringList& paths )
+void FolderWatcher::changeDetected(const QStringList &paths)
 {
-    // qDebug() << Q_FUNC_INFO << paths;
-
     // TODO: this shortcut doesn't look very reliable:
     //   - why is the timeout only 1 second?
     //   - what if there is more than one file being updated frequently?
@@ -80,7 +83,7 @@ void FolderWatcher::changeDetected( const QStringList& paths )
 
     // Check if the same path was reported within the last second.
     QSet<QString> pathsSet = paths.toSet();
-    if( pathsSet == _lastPaths && _timer.elapsed() < 1000 ) {
+    if (pathsSet == _lastPaths && _timer.elapsed() < 1000) {
         // the same path was reported within the last second. Skip.
         return;
     }
@@ -92,7 +95,7 @@ void FolderWatcher::changeDetected( const QStringList& paths )
     // ------- handle ignores:
     for (int i = 0; i < paths.size(); ++i) {
         QString path = paths[i];
-        if( pathIsIgnored(path) ) {
+        if (pathIsIgnored(path)) {
             continue;
         }
 
@@ -102,22 +105,21 @@ void FolderWatcher::changeDetected( const QStringList& paths )
         return;
     }
 
-    qDebug() << "detected changes in paths:" << changedPaths;
+    qCInfo(lcFolderWatcher) << "Detected changes in paths:" << changedPaths;
     foreach (const QString &path, changedPaths) {
         emit pathChanged(path);
     }
 }
 
-void FolderWatcher::addPath(const QString &path )
+void FolderWatcher::addPath(const QString &path)
 {
     _d->addPath(path);
 }
 
-void FolderWatcher::removePath(const QString &path )
+void FolderWatcher::removePath(const QString &path)
 {
     _d->removePath(path);
 }
 
 
 } // namespace OCC
-
