@@ -3,7 +3,8 @@
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; version 2 of the License.
+ * the Free Software Foundation; either version 2 of the License, or
+ * (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful, but
  * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
@@ -20,8 +21,7 @@
 #include <CoreServices/CoreServices.h>
 
 
-namespace OCC
-{
+namespace OCC {
 
 /**
  * @brief Mac OS X API implementation of FolderWatcher
@@ -30,7 +30,6 @@ namespace OCC
 class FolderWatcherPrivate
 {
 public:
-
     FolderWatcherPrivate(FolderWatcher *p, const QString &path);
     ~FolderWatcherPrivate();
 
@@ -47,7 +46,6 @@ private:
 
     FSEventStreamRef _stream;
 };
-
 }
 
 #endif
