@@ -3,7 +3,8 @@
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; version 2 of the License.
+ * the Free Software Foundation; either version 2 of the License, or
+ * (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful, but
  * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
@@ -12,7 +13,6 @@
  */
 
 #include <QThread>
-#include <QDebug>
 #include <QDir>
 
 #include "filesystem.h"
@@ -26,25 +26,23 @@
 namespace OCC {
 
 void WatcherThread::watchChanges(size_t fileNotifyBufferSize,
-                                 bool* increaseBufferSize)
+    bool *increaseBufferSize)
 {
     *increaseBufferSize = false;
     QString longPath = FileSystem::longWinPath(_path);
 
     _directory = CreateFileW(
-        (wchar_t*) longPath.utf16(),
+        (wchar_t *)longPath.utf16(),
         FILE_LIST_DIRECTORY,
         FILE_SHARE_WRITE | FILE_SHARE_READ | FILE_SHARE_DELETE,
         NULL,
         OPEN_EXISTING,
         FILE_FLAG_BACKUP_SEMANTICS | FILE_FLAG_OVERLAPPED,
-        NULL
-        );
+        NULL);
 
-    if (_directory == INVALID_HANDLE_VALUE)
-    {
+    if (_directory == INVALID_HANDLE_VALUE) {
         DWORD errorCode = GetLastError();
-        qDebug() << Q_FUNC_INFO << "Failed to create handle for" << _path << ", error:" << errorCode;
+        qCWarning(lcFolderWatcher) << "Failed to create handle for" << _path << ", error:" << errorCode;
         _directory = 0;
         return;
     }
@@ -53,7 +51,7 @@ void WatcherThread::watchChanges(size_t fileNotifyBufferSize,
     overlapped.hEvent = _resultEvent;
 
     // QVarLengthArray ensures the stack-buffer is aligned like double and qint64.
-    QVarLengthArray<char, 4096*10> fileNotifyBuffer;
+    QVarLengthArray<char, 4096 * 10> fileNotifyBuffer;
     fileNotifyBuffer.resize(fileNotifyBufferSize);
 
     const size_t fileNameBufferSize = 4096;
@@ -64,52 +62,49 @@ void WatcherThread::watchChanges(size_t fileNotifyBufferSize,
         ResetEvent(_resultEvent);
 
         FILE_NOTIFY_INFORMATION *pFileNotifyBuffer =
-                (FILE_NOTIFY_INFORMATION*)fileNotifyBuffer.data();
+            (FILE_NOTIFY_INFORMATION *)fileNotifyBuffer.data();
         DWORD dwBytesReturned = 0;
         SecureZeroMemory(pFileNotifyBuffer, fileNotifyBufferSize);
-        if(! ReadDirectoryChangesW( _directory, (LPVOID)pFileNotifyBuffer,
-                                    fileNotifyBufferSize, true,
-                                    FILE_NOTIFY_CHANGE_FILE_NAME |
-                                    FILE_NOTIFY_CHANGE_DIR_NAME |
-                                    FILE_NOTIFY_CHANGE_LAST_WRITE,
-                                    &dwBytesReturned,
-                                    &overlapped,
-                                    NULL))
-        {
+        if (!ReadDirectoryChangesW(_directory, (LPVOID)pFileNotifyBuffer,
+                fileNotifyBufferSize, true,
+                FILE_NOTIFY_CHANGE_FILE_NAME | FILE_NOTIFY_CHANGE_DIR_NAME | FILE_NOTIFY_CHANGE_LAST_WRITE,
+                &dwBytesReturned,
+                &overlapped,
+                NULL)) {
             DWORD errorCode = GetLastError();
             if (errorCode == ERROR_NOTIFY_ENUM_DIR) {
-                qDebug() << Q_FUNC_INFO << "The buffer for changes overflowed! Triggering a generic change and resizing";
+                qCDebug(lcFolderWatcher) << "The buffer for changes overflowed! Triggering a generic change and resizing";
                 emit changed(_path);
                 *increaseBufferSize = true;
             } else {
-                qDebug() << Q_FUNC_INFO << "ReadDirectoryChangesW error" << errorCode;
+                qCWarning(lcFolderWatcher) << "ReadDirectoryChangesW error" << errorCode;
             }
             break;
         }
 
-        HANDLE handles[] = {_resultEvent, _stopEvent};
+        HANDLE handles[] = { _resultEvent, _stopEvent };
         DWORD result = WaitForMultipleObjects(
-                    2, handles,
-                    false, // awake once one of them arrives
-                    INFINITE);
+            2, handles,
+            false, // awake once one of them arrives
+            INFINITE);
         if (result == 1) {
-            qDebug() << "Received stop event, aborting folder watcher thread";
+            qCDebug(lcFolderWatcher) << "Received stop event, aborting folder watcher thread";
             break;
         }
         if (result != 0) {
-            qDebug() << "WaitForMultipleObjects failed" << result << GetLastError();
+            qCWarning(lcFolderWatcher) << "WaitForMultipleObjects failed" << result << GetLastError();
             break;
         }
 
         bool ok = GetOverlappedResult(_directory, &overlapped, &dwBytesReturned, false);
-        if (! ok) {
+        if (!ok) {
             DWORD errorCode = GetLastError();
             if (errorCode == ERROR_NOTIFY_ENUM_DIR) {
-                qDebug() << Q_FUNC_INFO << "The buffer for changes overflowed! Triggering a generic change and resizing";
+                qCDebug(lcFolderWatcher) << "The buffer for changes overflowed! Triggering a generic change and resizing";
                 emit changed(_path);
                 *increaseBufferSize = true;
             } else {
-                qDebug() << Q_FUNC_INFO << "GetOverlappedResult error" << errorCode;
+                qCWarning(lcFolderWatcher) << "GetOverlappedResult error" << errorCode;
             }
             break;
         }
@@ -123,12 +118,12 @@ void WatcherThread::watchChanges(size_t fileNotifyBufferSize,
             // TODO: We could still try expanding the path in the tricky cases...
             QString longfile = file;
             if (curEntry->Action != FILE_ACTION_REMOVED
-                    && curEntry->Action != FILE_ACTION_RENAMED_OLD_NAME) {
+                && curEntry->Action != FILE_ACTION_RENAMED_OLD_NAME) {
                 size_t longNameSize = GetLongPathNameW(reinterpret_cast<LPCWSTR>(file.utf16()), fileNameBuffer, fileNameBufferSize);
                 if (longNameSize > 0) {
                     longfile = QString::fromUtf16(reinterpret_cast<const ushort *>(fileNameBuffer), longNameSize);
                 } else {
-                    qDebug() << Q_FUNC_INFO << "Error converting file name to full length, keeping original name.";
+                    qCWarning(lcFolderWatcher) << "Error converting file name to full length, keeping original name.";
                 }
             }
             longfile = QDir::cleanPath(longfile);
@@ -137,19 +132,16 @@ void WatcherThread::watchChanges(size_t fileNotifyBufferSize,
             // and new files in a folder, probably because of the folder's mtime
             // changing. We don't need them.
             bool skip = curEntry->Action == FILE_ACTION_MODIFIED
-                    && QFileInfo(longfile).isDir();
+                && QFileInfo(longfile).isDir();
 
             if (!skip) {
-                //qDebug() << Q_FUNC_INFO << "Found change in" << longfile
-                //         << "action:" << curEntry->Action;
                 emit changed(longfile);
             }
 
             if (curEntry->NextEntryOffset == 0) {
                 break;
             }
-            curEntry = (FILE_NOTIFY_INFORMATION*)(
-                            (char*)curEntry + curEntry->NextEntryOffset);
+            curEntry = (FILE_NOTIFY_INFORMATION *)((char *)curEntry + curEntry->NextEntryOffset);
         }
     }
 
@@ -172,15 +164,15 @@ void WatcherThread::run()
 
     // If this buffer fills up before we've extracted its data we will lose
     // change information. Therefore start big.
-    size_t bufferSize = 4096*10;
-    size_t maxBuffer = 64*1024;
+    size_t bufferSize = 4096 * 10;
+    size_t maxBuffer = 64 * 1024;
 
     while (!_done) {
         bool increaseBufferSize = false;
         watchChanges(bufferSize, &increaseBufferSize);
 
         if (increaseBufferSize) {
-            bufferSize = qMin(bufferSize*2, maxBuffer);
+            bufferSize = qMin(bufferSize * 2, maxBuffer);
         } else if (!_done) {
             // Other errors shouldn't actually happen,
             // so sleep a bit to avoid running into the same error case in a
@@ -201,12 +193,12 @@ void WatcherThread::stop()
     SetEvent(_stopEvent);
 }
 
-FolderWatcherPrivate::FolderWatcherPrivate(FolderWatcher *p, const QString& path)
+FolderWatcherPrivate::FolderWatcherPrivate(FolderWatcher *p, const QString &path)
     : _parent(p)
 {
     _thread = new WatcherThread(path);
-    connect(_thread, SIGNAL(changed(const QString&)),
-            _parent,SLOT(changeDetected(const QString&)));
+    connect(_thread, SIGNAL(changed(const QString &)),
+        _parent, SLOT(changeDetected(const QString &)));
     _thread->start();
 }
 
