@@ -3,7 +3,8 @@
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; version 2 of the License.
+ * the Free Software Foundation; either version 2 of the License, or
+ * (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful, but
  * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
@@ -17,7 +18,6 @@
 #include "theme.h"
 #include "configfile.h"
 #include "application.h"
-#include "utility.h"
 #include "configfile.h"
 #include "owncloudsetupwizard.h"
 #include "accountmanager.h"
@@ -31,20 +31,22 @@
 
 #include <QNetworkProxy>
 #include <QDir>
+#include <QScopedValueRollback>
 
 namespace OCC {
 
-GeneralSettings::GeneralSettings(QWidget *parent) :
-    QWidget(parent),
-    _ui(new Ui::GeneralSettings)
+GeneralSettings::GeneralSettings(QWidget *parent)
+    : QWidget(parent)
+    , _ui(new Ui::GeneralSettings)
+    , _currentlyLoading(false)
 {
     _ui->setupUi(this);
 
-    connect(_ui->desktopNotificationsCheckBox, SIGNAL(toggled(bool)),
-            SLOT(slotToggleOptionalDesktopNotifications(bool)));
+    connect(_ui->desktopNotificationsCheckBox, &QAbstractButton::toggled,
+        this, &GeneralSettings::slotToggleOptionalDesktopNotifications);
 
     _ui->autostartCheckBox->setChecked(Utility::hasLaunchOnStartup(Theme::instance()->appName()));
-    connect(_ui->autostartCheckBox, SIGNAL(toggled(bool)), SLOT(slotToggleLaunchOnStartup(bool)));
+    connect(_ui->autostartCheckBox, &QAbstractButton::toggled, this, &GeneralSettings::slotToggleLaunchOnStartup);
 
     // setup about section
     QString about = Theme::instance()->about();
@@ -61,10 +63,11 @@ GeneralSettings::GeneralSettings(QWidget *parent) :
     slotUpdateInfo();
 
     // misc
-    connect(_ui->monoIconsCheckBox, SIGNAL(toggled(bool)), SLOT(saveMiscSettings()));
-    connect(_ui->crashreporterCheckBox, SIGNAL(toggled(bool)), SLOT(saveMiscSettings()));
-    connect(_ui->newFolderLimitCheckBox, SIGNAL(toggled(bool)), SLOT(saveMiscSettings()));
-    connect(_ui->newFolderLimitSpinBox, SIGNAL(valueChanged(int)), SLOT(saveMiscSettings()));
+    connect(_ui->monoIconsCheckBox, &QAbstractButton::toggled, this, &GeneralSettings::saveMiscSettings);
+    connect(_ui->crashreporterCheckBox, &QAbstractButton::toggled, this, &GeneralSettings::saveMiscSettings);
+    connect(_ui->newFolderLimitCheckBox, &QAbstractButton::toggled, this, &GeneralSettings::saveMiscSettings);
+    connect(_ui->newFolderLimitSpinBox, static_cast<void (QSpinBox::*)(int)>(&QSpinBox::valueChanged), this, &GeneralSettings::saveMiscSettings);
+    connect(_ui->newExternalStorage, &QAbstractButton::toggled, this, &GeneralSettings::saveMiscSettings);
 
 #ifndef WITH_CRASHREPORTER
     _ui->crashreporterCheckBox->setVisible(false);
@@ -74,16 +77,17 @@ GeneralSettings::GeneralSettings(QWidget *parent) :
      * align properly vertically , fixes bug #3758
      */
     int m0, m1, m2, m3;
-    _ui->horizontalLayout_3->getContentsMargins( &m0, &m1, &m2, &m3 );
-    _ui->horizontalLayout_3->setContentsMargins(0, m1, m2, m3 );
+    _ui->horizontalLayout_3->getContentsMargins(&m0, &m1, &m2, &m3);
+    _ui->horizontalLayout_3->setContentsMargins(0, m1, m2, m3);
 
     // OEM themes are not obliged to ship mono icons, so there
     // is no point in offering an option
-    QString themeDir = QString::fromLatin1(":/client/theme/%1/")
-            .arg(Theme::instance()->systrayIconFlavor(true));
-    _ui->monoIconsCheckBox->setVisible(QDir(themeDir).exists());
+    _ui->monoIconsCheckBox->setVisible(Theme::instance()->monoIconsAvailable());
 
-    connect(_ui->ignoredFilesButton, SIGNAL(clicked()), SLOT(slotIgnoreFilesEditor()));
+    connect(_ui->ignoredFilesButton, &QAbstractButton::clicked, this, &GeneralSettings::slotIgnoreFilesEditor);
+
+    // accountAdded means the wizard was finished and the wizard might change some options.
+    connect(AccountManager::instance(), &AccountManager::accountAdded, this, &GeneralSettings::loadMiscSettings);
 }
 
 GeneralSettings::~GeneralSettings()
@@ -92,12 +96,14 @@ GeneralSettings::~GeneralSettings()
     delete _syncLogDialog;
 }
 
-QSize GeneralSettings::sizeHint() const {
+QSize GeneralSettings::sizeHint() const
+{
     return QSize(ownCloudGui::settingsDialogSize().width(), QWidget::sizeHint().height());
 }
 
 void GeneralSettings::loadMiscSettings()
 {
+    QScopedValueRollback<bool> scope(_currentlyLoading, true);
     ConfigFile cfgFile;
     _ui->monoIconsCheckBox->setChecked(cfgFile.monoIcons());
     _ui->desktopNotificationsCheckBox->setChecked(cfgFile.optionalDesktopNotifications());
@@ -105,19 +111,22 @@ void GeneralSettings::loadMiscSettings()
     auto newFolderLimit = cfgFile.newBigFolderSizeLimit();
     _ui->newFolderLimitCheckBox->setChecked(newFolderLimit.first);
     _ui->newFolderLimitSpinBox->setValue(newFolderLimit.second);
+    _ui->newExternalStorage->setChecked(cfgFile.confirmExternalStorage());
+    _ui->monoIconsCheckBox->setChecked(cfgFile.monoIcons());
 }
 
 void GeneralSettings::slotUpdateInfo()
 {
-    OCUpdater *updater = dynamic_cast<OCUpdater*>(Updater::instance());
+    // Note: the sparkle-updater is not an OCUpdater
+    OCUpdater *updater = qobject_cast<OCUpdater *>(Updater::instance());
     if (ConfigFile().skipUpdateCheck()) {
         updater = 0; // don't show update info if updates are disabled
     }
 
     if (updater) {
-        connect(updater, SIGNAL(downloadStateChanged()), SLOT(slotUpdateInfo()), Qt::UniqueConnection);
-        connect(_ui->restartButton, SIGNAL(clicked()), updater, SLOT(slotStartInstaller()), Qt::UniqueConnection);
-        connect(_ui->restartButton, SIGNAL(clicked()), qApp, SLOT(quit()), Qt::UniqueConnection);
+        connect(updater, &OCUpdater::downloadStateChanged, this, &GeneralSettings::slotUpdateInfo, Qt::UniqueConnection);
+        connect(_ui->restartButton, &QAbstractButton::clicked, updater, &OCUpdater::slotStartInstaller, Qt::UniqueConnection);
+        connect(_ui->restartButton, &QAbstractButton::clicked, qApp, &QApplication::quit, Qt::UniqueConnection);
         _ui->updateStateLabel->setText(updater->statusString());
         _ui->restartButton->setVisible(updater->downloadState() == OCUpdater::DownloadComplete);
     } else {
@@ -128,6 +137,8 @@ void GeneralSettings::slotUpdateInfo()
 
 void GeneralSettings::saveMiscSettings()
 {
+    if (_currentlyLoading)
+        return;
     ConfigFile cfgFile;
     bool isChecked = _ui->monoIconsCheckBox->isChecked();
     cfgFile.setMonoIcons(isChecked);
@@ -135,7 +146,8 @@ void GeneralSettings::saveMiscSettings()
     cfgFile.setCrashReporter(_ui->crashreporterCheckBox->isChecked());
 
     cfgFile.setNewBigFolderSizeLimit(_ui->newFolderLimitCheckBox->isChecked(),
-                                        _ui->newFolderLimitSpinBox->value());
+        _ui->newFolderLimitSpinBox->value());
+    cfgFile.setConfirmExternalStorage(_ui->newExternalStorage->isChecked());
 }
 
 void GeneralSettings::slotToggleLaunchOnStartup(bool enable)
@@ -154,7 +166,7 @@ void GeneralSettings::slotIgnoreFilesEditor()
 {
     if (_ignoreEditor.isNull()) {
         _ignoreEditor = new IgnoreListEditor(this);
-        _ignoreEditor->setAttribute( Qt::WA_DeleteOnClose, true );
+        _ignoreEditor->setAttribute(Qt::WA_DeleteOnClose, true);
         _ignoreEditor->open();
     } else {
         ownCloudGui::raiseDialog(_ignoreEditor);
