@@ -3,7 +3,8 @@
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; version 2 of the License.
+ * the Free Software Foundation; either version 2 of the License, or
+ * (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful, but
  * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
@@ -21,9 +22,13 @@ class QListWidgetItem;
 namespace OCC {
 
 namespace Ui {
-class IgnoreListEditor;
+    class IgnoreListEditor;
 }
 
+/**
+ * @brief The IgnoreListEditor class
+ * @ingroup gui
+ */
 class IgnoreListEditor : public QDialog
 {
     Q_OBJECT
@@ -32,15 +37,18 @@ public:
     explicit IgnoreListEditor(QWidget *parent = 0);
     ~IgnoreListEditor();
 
+    bool ignoreHiddenFiles();
+
 private slots:
     void slotItemSelectionChanged();
     void slotRemoveCurrentItem();
     void slotUpdateLocalIgnoreList();
     void slotAddPattern();
-    void slotEditPattern(QListWidgetItem*);
 
 private:
-    void readIgnoreFile(const QString& file, bool readOnly);
+    void readIgnoreFile(const QString &file, bool readOnly);
+    int addPattern(const QString &pattern, bool deletable, bool readOnly);
+    QString readOnlyTooltip;
     Ui::IgnoreListEditor *ui;
 };
 
