@@ -3,7 +3,8 @@
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; version 2 of the License.
+ * the Free Software Foundation; either version 2 of the License, or
+ * (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful, but
  * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
@@ -22,8 +23,8 @@
 #include <QPointer>
 #include <QAction>
 #include <QMenu>
-#include <QSignalMapper>
 #include <QSize>
+#include <QTimer>
 
 namespace OCC {
 
@@ -52,45 +53,61 @@ public:
     static QSize settingsDialogSize() { return QSize(800, 500); }
     void setupOverlayIcons();
 
+    /// Whether the tray menu is visible
+    bool contextMenuVisible() const;
+
 signals:
     void setupProxy();
 
 public slots:
     void setupContextMenu();
+    void updateContextMenu();
+    void updateContextMenuNeeded();
+    void slotContextMenuAboutToShow();
+    void slotContextMenuAboutToHide();
     void slotComputeOverallSyncStatus();
     void slotShowTrayMessage(const QString &title, const QString &msg);
     void slotShowOptionalTrayMessage(const QString &title, const QString &msg);
-    void slotFolderOpenAction( const QString& alias );
+    void slotFolderOpenAction(const QString &alias);
     void slotRebuildRecentMenus();
-    void slotUpdateProgress(const QString &folder, const ProgressInfo& progress);
+    void slotUpdateProgress(const QString &folder, const ProgressInfo &progress);
     void slotShowGuiMessage(const QString &title, const QString &message);
     void slotFoldersChanged();
     void slotShowSettings();
     void slotShowSyncProtocol();
     void slotShutdown();
-    void slotSyncStateChange(Folder*);
-    void slotTrayClicked( QSystemTrayIcon::ActivationReason reason );
+    void slotSyncStateChange(Folder *);
+    void slotTrayClicked(QSystemTrayIcon::ActivationReason reason);
     void slotToggleLogBrowser();
     void slotOpenOwnCloud();
     void slotOpenSettingsDialog();
     void slotHelp();
-    void slotOpenPath(const QString& path);
+    void slotOpenPath(const QString &path);
     void slotAccountStateChanged();
     void slotTrayMessageIfServerUnsupported(Account *account);
-    void slotShowShareDialog(const QString &sharePath, const QString &localPath, bool resharingAllowed);
+
+    /**
+     * Open a share dialog for a file or folder.
+     *
+     * sharePath is the full remote path to the item,
+     * localPath is the absolute local path to it (so not relative
+     * to the folder).
+     */
+    void slotShowShareDialog(const QString &sharePath, const QString &localPath);
+
     void slotRemoveDestroyedShareDialogs();
 
 private slots:
-    void slotDisplayIdle();
     void slotLogin();
     void slotLogout();
     void slotUnpauseAllFolders();
     void slotPauseAllFolders();
+    void slotNewAccountWizard();
 
 private:
     void setPauseOnAllFoldersHelper(bool pause);
     void setupActions();
-    void addAccountContextMenu(AccountStatePtr accountState, QMenu* menu, bool separateMenu);
+    void addAccountContextMenu(AccountStatePtr accountState, QMenu *menu, bool separateMenu);
 
     QPointer<Systray> _tray;
 #if defined(Q_OS_MAC)
@@ -98,17 +115,24 @@ private:
 #else
     QPointer<SettingsDialog> _settingsDialog;
 #endif
-    QPointer<LogBrowser>_logBrowser;
-       // tray's menu
+    QPointer<LogBrowser> _logBrowser;
+    // tray's menu
     QScopedPointer<QMenu> _contextMenu;
+
+    // Manually tracking whether the context menu is visible, but only works
+    // on OSX because aboutToHide is not reliable everywhere.
+    bool _contextMenuVisibleOsx;
+
     QMenu *_recentActionsMenu;
-    QVector<QMenu*> _accountMenus;
+    QVector<QMenu *> _accountMenus;
     bool _qdbusmenuWorkaround;
-    QMap<QString, QPointer<ShareDialog> > _shareDialogs;
+    QTimer _workaroundBatchTrayUpdate;
+    QMap<QString, QPointer<ShareDialog>> _shareDialogs;
 
     QAction *_actionLogin;
     QAction *_actionLogout;
 
+    QAction *_actionNewAccountWizard;
     QAction *_actionSettings;
     QAction *_actionStatus;
     QAction *_actionEstimate;
@@ -117,11 +141,7 @@ private:
     QAction *_actionQuit;
     QAction *_actionCrash;
 
-    QList<QAction*> _recentItemsActions;
-
-    QSignalMapper *_folderOpenActionMapper;
-    QSignalMapper *_recentItemsMapper;
-
+    QList<QAction *> _recentItemsActions;
     Application *_app;
 };
 
