@@ -3,7 +3,8 @@
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; version 2 of the License.
+ * the Free Software Foundation; either version 2 of the License, or
+ * (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful, but
  * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
@@ -12,6 +13,7 @@
  */
 
 #include "sharedialog.h"
+#include "sharee.h"
 #include "sharelinkwidget.h"
 #include "shareusergroupwidget.h"
 #include "ui_sharedialog.h"
@@ -24,38 +26,42 @@
 
 #include <QFileInfo>
 #include <QFileIconProvider>
-#include <QDebug>
 #include <QPointer>
 #include <QPushButton>
 #include <QFrame>
 
 namespace OCC {
 
+static const int thumbnailSize = 40;
+
 ShareDialog::ShareDialog(QPointer<AccountState> accountState,
-                         const QString &sharePath,
-                         const QString &localPath,
-                         SharePermissions maxSharingPermissions,
-                         QWidget *parent) :
-    QDialog(parent),
-    _ui(new Ui::ShareDialog),
-    _accountState(accountState),
-    _sharePath(sharePath),
-    _localPath(localPath),
-    _maxSharingPermissions(maxSharingPermissions),
-    _linkWidget(NULL),
-    _userGroupWidget(NULL),
-    _progressIndicator(NULL)
+    const QString &sharePath,
+    const QString &localPath,
+    SharePermissions maxSharingPermissions,
+    const QByteArray &numericFileId,
+    QWidget *parent)
+    : QDialog(parent)
+    , _ui(new Ui::ShareDialog)
+    , _accountState(accountState)
+    , _sharePath(sharePath)
+    , _localPath(localPath)
+    , _maxSharingPermissions(maxSharingPermissions)
+    , _privateLinkUrl(accountState->account()->deprecatedPrivateLinkUrl(numericFileId).toString(QUrl::FullyEncoded))
+    , _linkWidget(NULL)
+    , _userGroupWidget(NULL)
+    , _progressIndicator(NULL)
 {
+    setWindowFlags(windowFlags() & ~Qt::WindowContextHelpButtonHint);
     setAttribute(Qt::WA_DeleteOnClose);
     setObjectName("SharingDialog"); // required as group for saveGeometry call
 
     _ui->setupUi(this);
 
     QPushButton *closeButton = _ui->buttonBox->button(QDialogButtonBox::Close);
-    connect(closeButton, SIGNAL(clicked()), this, SLOT(close()));
+    connect(closeButton, &QAbstractButton::clicked, this, &QWidget::close);
 
     // We want to act on account state changes
-    connect(_accountState, SIGNAL(stateChanged(int)), SLOT(slotAccountStateChanged(int)));
+    connect(_accountState.data(), &AccountState::stateChanged, this, &ShareDialog::slotAccountStateChanged);
 
     // Because people press enter in the dialog and we don't want to close for that
     closeButton->setDefault(false);
@@ -65,19 +71,24 @@ ShareDialog::ShareDialog(QPointer<AccountState> accountState,
     QFileInfo f_info(_localPath);
     QFileIconProvider icon_provider;
     QIcon icon = icon_provider.icon(f_info);
-    _ui->label_icon->setPixmap(icon.pixmap(40,40));
+    auto pixmap = icon.pixmap(thumbnailSize, thumbnailSize);
+    if (pixmap.width() > 0) {
+        _ui->label_icon->setPixmap(pixmap);
+    } else {
+        _ui->label_icon->hide();
+    }
 
     // Set filename
     QFileInfo lPath(_localPath);
     QString fileName = lPath.fileName();
     _ui->label_name->setText(tr("%1").arg(fileName));
-    QFont f( _ui->label_name->font());
-    f.setPointSize( f.pointSize() * 1.4 );
-    _ui->label_name->setFont( f );
+    QFont f(_ui->label_name->font());
+    f.setPointSize(f.pointSize() * 1.4);
+    _ui->label_name->setFont(f);
 
     _ui->label_sharePath->setWordWrap(true);
     QString ocDir(_sharePath);
-    ocDir.truncate(ocDir.length()-fileName.length());
+    ocDir.truncate(ocDir.length() - fileName.length());
 
     ocDir.replace(QRegExp("^/*"), "");
     ocDir.replace(QRegExp("/*$"), "");
@@ -86,7 +97,7 @@ ShareDialog::ShareDialog(QPointer<AccountState> accountState,
     // may be in use or not.
     _ui->gridLayout->removeWidget(_ui->label_sharePath);
     _ui->gridLayout->removeWidget(_ui->label_name);
-    if( ocDir.isEmpty() ) {
+    if (ocDir.isEmpty()) {
         _ui->gridLayout->addWidget(_ui->label_name, 0, 1, 2, 1);
         _ui->label_sharePath->setText(QString());
     } else {
@@ -98,28 +109,35 @@ ShareDialog::ShareDialog(QPointer<AccountState> accountState,
     this->setWindowTitle(tr("%1 Sharing").arg(Theme::instance()->appNameGUI()));
 
     if (!accountState->account()->capabilities().shareAPI()) {
-        _ui->shareWidgetsLayout->addWidget(new QLabel(tr("The server does not allow sharing")));
+        auto label = new QLabel(tr("The server does not allow sharing"));
+        label->setWordWrap(true);
+        label->setSizePolicy(QSizePolicy::Expanding, QSizePolicy::Expanding);
+        layout()->replaceWidget(_ui->shareWidgets, label);
         return;
     }
 
     if (QFileInfo(_localPath).isFile()) {
         ThumbnailJob *job = new ThumbnailJob(_sharePath, _accountState->account(), this);
-        connect(job, SIGNAL(jobFinished(int, QByteArray)), SLOT(slotThumbnailFetched(int, QByteArray)));
+        connect(job, &ThumbnailJob::jobFinished, this, &ShareDialog::slotThumbnailFetched);
         job->start();
     }
 
     _progressIndicator = new QProgressIndicator(this);
     _progressIndicator->startAnimation();
     _progressIndicator->setToolTip(tr("Retrieving maximum possible sharing permissions from server..."));
-    _ui->shareWidgetsLayout->addWidget(_progressIndicator);
+    _ui->buttonBoxLayout->insertWidget(0, _progressIndicator);
 
     // Server versions >= 9.1 support the "share-permissions" property
     // older versions will just return share-permissions: ""
     auto job = new PropfindJob(accountState->account(), _sharePath);
-    job->setProperties(QList<QByteArray>() << "http://open-collaboration-services.org/ns:share-permissions");
+    job->setProperties(
+        QList<QByteArray>()
+        << "http://open-collaboration-services.org/ns:share-permissions"
+        << "http://owncloud.org/ns:fileid" // numeric file id for fallback private link generation
+        << "http://owncloud.org/ns:privatelink");
     job->setTimeout(10 * 1000);
-    connect(job, SIGNAL(result(QVariantMap)), SLOT(slotMaxSharingPermissionsReceived(QVariantMap)));
-    connect(job, SIGNAL(finishedWithError(QNetworkReply*)), SLOT(slotMaxSharingPermissionsError()));
+    connect(job, &PropfindJob::result, this, &ShareDialog::slotPropfindReceived);
+    connect(job, &PropfindJob::finishedWithError, this, &ShareDialog::slotPropfindError);
     job->start();
 }
 
@@ -128,24 +146,34 @@ ShareDialog::~ShareDialog()
     delete _ui;
 }
 
-void ShareDialog::done( int r ) {
+void ShareDialog::done(int r)
+{
     ConfigFile cfg;
     cfg.saveGeometry(this);
     QDialog::done(r);
 }
 
-void ShareDialog::slotMaxSharingPermissionsReceived(const QVariantMap & result)
+void ShareDialog::slotPropfindReceived(const QVariantMap &result)
 {
     const QVariant receivedPermissions = result["share-permissions"];
     if (!receivedPermissions.toString().isEmpty()) {
         _maxSharingPermissions = static_cast<SharePermissions>(receivedPermissions.toInt());
-        qDebug() << "Received sharing permissions for" << _sharePath << _maxSharingPermissions;
+        qCInfo(lcSharing) << "Received sharing permissions for" << _sharePath << _maxSharingPermissions;
+    }
+    auto privateLinkUrl = result["privatelink"].toString();
+    auto numericFileId = result["fileid"].toByteArray();
+    if (!privateLinkUrl.isEmpty()) {
+        qCInfo(lcSharing) << "Received private link url for" << _sharePath << privateLinkUrl;
+        _privateLinkUrl = privateLinkUrl;
+    } else if (!numericFileId.isEmpty()) {
+        qCInfo(lcSharing) << "Received numeric file id for" << _sharePath << numericFileId;
+        _privateLinkUrl = _accountState->account()->deprecatedPrivateLinkUrl(numericFileId).toString(QUrl::FullyEncoded);
     }
 
     showSharingUi();
 }
 
-void ShareDialog::slotMaxSharingPermissionsError()
+void ShareDialog::slotPropfindError()
 {
     // On error show the share ui anyway. The user can still see shares,
     // delete them and so on, even though adding new shares or granting
@@ -167,37 +195,27 @@ void ShareDialog::showSharingUi()
     if (!canReshare) {
         auto label = new QLabel(this);
         label->setText(tr("The file can not be shared because it was shared without sharing permission."));
-        _ui->shareWidgetsLayout->addWidget(label);
+        label->setWordWrap(true);
+        label->setSizePolicy(QSizePolicy::Expanding, QSizePolicy::Expanding);
+        layout()->replaceWidget(_ui->shareWidgets, label);
         return;
     }
 
     // We only do user/group sharing from 8.2.0
     bool userGroupSharing =
-            theme->userGroupSharing()
-            && _accountState->account()->serverVersionInt() >= ((8 << 16) + (2 << 8));
-
-    bool autoShare = !userGroupSharing;
+        theme->userGroupSharing()
+        && _accountState->account()->serverVersionInt() >= Account::makeServerVersion(8, 2, 0);
 
     if (userGroupSharing) {
-        _userGroupWidget = new ShareUserGroupWidget(_accountState->account(), _sharePath, _localPath, _maxSharingPermissions, this);
-        _ui->shareWidgetsLayout->addWidget(_userGroupWidget);
+        _userGroupWidget = new ShareUserGroupWidget(_accountState->account(), _sharePath, _localPath, _maxSharingPermissions, _privateLinkUrl, this);
+        _ui->shareWidgets->addTab(_userGroupWidget, tr("Users and Groups"));
         _userGroupWidget->getShares();
     }
 
     if (theme->linkSharing()) {
-        if (userGroupSharing) {
-            QFrame *hline = new QFrame(this);
-            hline->setFrameShape(QFrame::HLine);
-            QPalette p = palette();
-            // Make the line softer:
-            p.setColor(QPalette::Foreground, QColor::fromRgba((p.color(QPalette::Foreground).rgba() & 0x00ffffff) | 0x50000000));
-            hline->setPalette(p);
-            _ui->shareWidgetsLayout->addWidget(hline);
-        }
-
-        _linkWidget = new ShareLinkWidget(_accountState->account(), _sharePath, _localPath, _maxSharingPermissions, autoShare, this);
+        _linkWidget = new ShareLinkWidget(_accountState->account(), _sharePath, _localPath, _maxSharingPermissions, this);
         _linkWidget->setSizePolicy(QSizePolicy::MinimumExpanding, QSizePolicy::Preferred);
-        _ui->shareWidgetsLayout->addWidget(_linkWidget);
+        _ui->shareWidgets->addTab(_linkWidget, tr("Public Links"));
         _linkWidget->getShares();
     }
 }
@@ -205,18 +223,21 @@ void ShareDialog::showSharingUi()
 void ShareDialog::slotThumbnailFetched(const int &statusCode, const QByteArray &reply)
 {
     if (statusCode != 200) {
-        qDebug() << Q_FUNC_INFO << "Status code: " << statusCode;
+        qCWarning(lcSharing) << "Thumbnail status code: " << statusCode;
         return;
     }
 
     QPixmap p;
     p.loadFromData(reply, "PNG");
+    p = p.scaledToHeight(thumbnailSize, Qt::SmoothTransformation);
     _ui->label_icon->setPixmap(p);
+    _ui->label_icon->show();
 }
 
-void ShareDialog::slotAccountStateChanged(int state) {
+void ShareDialog::slotAccountStateChanged(int state)
+{
     bool enabled = (state == AccountState::State::Connected);
-    qDebug() << Q_FUNC_INFO << enabled;
+    qCDebug(lcSharing) << "Account connected?" << enabled;
 
     if (_userGroupWidget != NULL) {
         _userGroupWidget->setEnabled(enabled);
@@ -226,6 +247,4 @@ void ShareDialog::slotAccountStateChanged(int state) {
         _linkWidget->setEnabled(enabled);
     }
 }
-
 }
-
