@@ -4,7 +4,8 @@
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; version 2 of the License.
+ * the Free Software Foundation; either version 2 of the License, or
+ * (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful, but
  * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
@@ -26,27 +27,25 @@
 
 namespace OCC {
 
-SyncLogDialog::SyncLogDialog(QWidget *parent, ProtocolWidget *protoWidget) :
-   QDialog(parent),
-    _ui(new Ui::SyncLogDialog)
+SyncLogDialog::SyncLogDialog(QWidget *parent, ProtocolWidget *protoWidget)
+    : QDialog(parent)
+    , _ui(new Ui::SyncLogDialog)
 {
     setObjectName("SyncLogDialog"); // required as group for saveGeometry call
 
     _ui->setupUi(this);
 
-    if( protoWidget) {
+    if (protoWidget) {
         _ui->logWidgetLayout->addWidget(protoWidget);
     }
 
     QPushButton *closeButton = _ui->buttonBox->button(QDialogButtonBox::Close);
-    if( closeButton ) {
-        connect( closeButton, SIGNAL(clicked()), this, SLOT(close()) );
+    if (closeButton) {
+        connect(closeButton, &QAbstractButton::clicked, this, &QWidget::close);
     }
 }
 
 SyncLogDialog::~SyncLogDialog()
 {
-
 }
-
 }
