@@ -3,7 +3,8 @@
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; version 2 of the License.
+ * the Free Software Foundation; either version 2 of the License, or
+ * (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful, but
  * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
@@ -20,15 +21,17 @@
 
 namespace OCC {
 
-class SparkleUpdater : public Updater {
+class SparkleUpdater : public Updater
+{
 public:
-    SparkleUpdater(const QString& appCastUrl);
+    SparkleUpdater(const QString &appCastUrl);
     ~SparkleUpdater();
 
     // unused in this updater
     void checkForUpdate() Q_DECL_OVERRIDE;
     void backgroundCheckForUpdate() Q_DECL_OVERRIDE;
     bool handleStartup() Q_DECL_OVERRIDE { return false; }
+
 private:
     class Private;
     Private *d;
