@@ -23,26 +23,25 @@
 #include "wizard/owncloudwizardcommon.h"
 #include "wizard/owncloudwizard.h"
 
-namespace OCC
-{
+namespace OCC {
 
-OwncloudHttpCredsPage::OwncloudHttpCredsPage(QWidget* parent)
-  : AbstractCredentialsWizardPage(),
-    _ui(),
-    _connected(false),
-    _progressIndi(new QProgressIndicator (this))
+OwncloudHttpCredsPage::OwncloudHttpCredsPage(QWidget *parent)
+    : AbstractCredentialsWizardPage()
+    , _ui()
+    , _connected(false)
+    , _progressIndi(new QProgressIndicator(this))
 {
     _ui.setupUi(this);
 
-    if(parent){
+    if (parent) {
         _ocWizard = qobject_cast<OwncloudWizard *>(parent);
     }
 
-    registerField( QLatin1String("OCUser*"),   _ui.leUsername);
-    registerField( QLatin1String("OCPasswd*"), _ui.lePassword);
+    registerField(QLatin1String("OCUser*"), _ui.leUsername);
+    registerField(QLatin1String("OCPasswd*"), _ui.lePassword);
 
     Theme *theme = Theme::instance();
-    switch(theme->userIDType()) {
+    switch (theme->userIDType()) {
     case Theme::UserIDUserName:
         // default, handled in ui file
         break;
@@ -60,7 +59,7 @@ OwncloudHttpCredsPage::OwncloudHttpCredsPage(QWidget* parent)
     setTitle(WizardCommon::titleTemplate().arg(tr("Connect to %1").arg(Theme::instance()->appNameGUI())));
     setSubTitle(WizardCommon::subTitleTemplate().arg(tr("Enter user credentials")));
 
-    _ui.resultLayout->addWidget( _progressIndi );
+    _ui.resultLayout->addWidget(_progressIndi);
     stopSpinner();
     setupCustomization();
 }
@@ -72,22 +71,22 @@ void OwncloudHttpCredsPage::setupCustomization()
     _ui.bottomLabel->hide();
 
     Theme *theme = Theme::instance();
-    QVariant variant = theme->customMedia( Theme::oCSetupTop );
-    if( !variant.isNull() ) {
-        WizardCommon::setupCustomMedia( variant, _ui.topLabel );
+    QVariant variant = theme->customMedia(Theme::oCSetupTop);
+    if (!variant.isNull()) {
+        WizardCommon::setupCustomMedia(variant, _ui.topLabel);
     }
 
-    variant = theme->customMedia( Theme::oCSetupBottom );
-    WizardCommon::setupCustomMedia( variant, _ui.bottomLabel );
+    variant = theme->customMedia(Theme::oCSetupBottom);
+    WizardCommon::setupCustomMedia(variant, _ui.bottomLabel);
 }
 
 void OwncloudHttpCredsPage::initializePage()
 {
     WizardCommon::initErrorLabel(_ui.errorLabel);
 
-    OwncloudWizard* ocWizard = qobject_cast< OwncloudWizard* >(wizard());
+    OwncloudWizard *ocWizard = qobject_cast<OwncloudWizard *>(wizard());
     AbstractCredentials *cred = ocWizard->account()->credentials();
-    HttpCredentials *httpCreds = qobject_cast<HttpCredentials*>(cred);
+    HttpCredentials *httpCreds = qobject_cast<HttpCredentials *>(cred);
     if (httpCreds) {
         const QString user = httpCreds->fetchUser();
         if (!user.isEmpty()) {
@@ -106,10 +105,10 @@ void OwncloudHttpCredsPage::initializePage()
         const QString user = url.userName();
         const QString password = url.password();
 
-        if(!user.isEmpty()) {
+        if (!user.isEmpty()) {
             _ui.leUsername->setText(user);
         }
-        if(!password.isEmpty()) {
+        if (!password.isEmpty()) {
             _ui.lePassword->setText(password);
         }
     }
@@ -135,7 +134,7 @@ bool OwncloudHttpCredsPage::validatePage()
         startSpinner();
 
         // Reset cookies to ensure the username / password is actually used
-        OwncloudWizard* ocWizard = qobject_cast< OwncloudWizard* >(wizard());
+        OwncloudWizard *ocWizard = qobject_cast<OwncloudWizard *>(wizard());
         ocWizard->account()->clearCookieJar();
 
         emit completeChanged();
@@ -161,7 +160,7 @@ int OwncloudHttpCredsPage::nextId() const
 void OwncloudHttpCredsPage::setConnected()
 {
     _connected = true;
-    stopSpinner ();
+    stopSpinner();
 }
 
 void OwncloudHttpCredsPage::startSpinner()
@@ -178,9 +177,9 @@ void OwncloudHttpCredsPage::stopSpinner()
     _progressIndi->stopAnimation();
 }
 
-void OwncloudHttpCredsPage::setErrorString(const QString& err)
+void OwncloudHttpCredsPage::setErrorString(const QString &err)
 {
-    if( err.isEmpty()) {
+    if (err.isEmpty()) {
         _ui.errorLabel->setVisible(false);
     } else {
         _ui.errorLabel->setVisible(true);
@@ -190,7 +189,7 @@ void OwncloudHttpCredsPage::setErrorString(const QString& err)
     stopSpinner();
 }
 
-AbstractCredentials* OwncloudHttpCredsPage::getCredentials() const
+AbstractCredentials *OwncloudHttpCredsPage::getCredentials() const
 {
     return new HttpCredentialsGui(_ui.leUsername->text(), _ui.lePassword->text(), _ocWizard->_clientSslCertificate, _ocWizard->_clientSslKey);
 }
