@@ -3,7 +3,8 @@
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; version 2 of the License.
+ * the Free Software Foundation; either version 2 of the License, or
+ * (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful, but
  * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
@@ -20,24 +21,23 @@
 class QByteArray;
 class QUrl;
 
-namespace OCC
-{
+namespace OCC {
 
+/**
+ * @brief The AccessManager class
+ * @ingroup libsync
+ */
 class OWNCLOUDSYNC_EXPORT AccessManager : public QNetworkAccessManager
 {
     Q_OBJECT
 
 public:
-    AccessManager(QObject* parent = 0);
+    AccessManager(QObject *parent = 0);
 
-    void setRawCookie(const  QByteArray &rawCookie, const  QUrl &url);
+    void setRawCookie(const QByteArray &rawCookie, const QUrl &url);
 
 protected:
-    QNetworkReply* createRequest(QNetworkAccessManager::Operation op, const QNetworkRequest& request, QIODevice* outgoingData = 0) Q_DECL_OVERRIDE;
-protected slots:
-    void slotProxyAuthenticationRequired(const QNetworkProxy &proxy, QAuthenticator *authenticator);
-    void slotAuthenticationRequired(QNetworkReply *reply, QAuthenticator *authenticator);
-
+    QNetworkReply *createRequest(QNetworkAccessManager::Operation op, const QNetworkRequest &request, QIODevice *outgoingData = 0) Q_DECL_OVERRIDE;
 };
 
 } // namespace OCC
