@@ -3,7 +3,8 @@
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; version 2 of the License.
+ * the Free Software Foundation; either version 2 of the License, or
+ * (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful, but
  * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
@@ -17,11 +18,12 @@
 #include "configfile.h"
 #include "accessmanager.h"
 #include "creds/abstractcredentials.h"
-#include "../3rdparty/certificates/p12topem.h"
 #include "capabilities.h"
 #include "theme.h"
+#include "common/asserts.h"
 
 #include <QSettings>
+#include <QLoggingCategory>
 #include <QMutex>
 #include <QNetworkReply>
 #include <QNetworkAccessManager>
@@ -29,17 +31,17 @@
 #include <QNetworkCookieJar>
 #include <QFileInfo>
 #include <QDir>
-#include <QDebug>
 #include <QSslKey>
+#include <QAuthenticator>
 
 namespace OCC {
 
+Q_LOGGING_CATEGORY(lcAccount, "sync.account", QtInfoMsg)
 
 Account::Account(QObject *parent)
     : QObject(parent)
     , _capabilities(QVariantMap())
-    , _davPath( Theme::instance()->webDavPath() )
-    , _wasMigrated(false)
+    , _davPath(Theme::instance()->webDavPath())
 {
     qRegisterMetaType<AccountPtr>("AccountPtr");
 }
@@ -57,8 +59,13 @@ Account::~Account()
 
 QString Account::davPath() const
 {
+    if (capabilities().chunkingNg()) {
+        // The chunking-ng means the server prefer to use the new webdav URL
+        return QLatin1String("/remote.php/dav/files/") + davUser() + QLatin1Char('/');
+    }
+
     // make sure to have a trailing slash
-    if( !_davPath.endsWith('/') ) {
+    if (!_davPath.endsWith('/')) {
         QString dp(_davPath);
         dp.append('/');
         return dp;
@@ -76,11 +83,31 @@ AccountPtr Account::sharedFromThis()
     return _sharedThis.toStrongRef();
 }
 
+QString Account::davUser() const
+{
+    return _davUser.isEmpty() ? _credentials->user() : _davUser;
+}
+
+void Account::setDavUser(const QString &newDavUser)
+{
+    _davUser = newDavUser;
+}
+
+#ifndef TOKEN_AUTH_ONLY
+QImage Account::avatar() const
+{
+    return _avatarImg;
+}
+void Account::setAvatar(const QImage &img)
+{
+    _avatarImg = img;
+    emit accountChangedAvatar();
+}
+#endif
 
 QString Account::displayName() const
 {
-    auto user = _credentials->user();
-    QString dn = QString("%1@%2").arg(user, _url.host());
+    QString dn = QString("%1@%2").arg(davUser(), _url.host());
     int port = url().port();
     if (port > 0 && port != 80 && port != 443) {
         dn.append(QLatin1Char(':'));
@@ -89,35 +116,22 @@ QString Account::displayName() const
     return dn;
 }
 
+QString Account::davDisplayName() const
+{
+    return _displayName;
+}
+
+void Account::setDavDisplayName(const QString &newDisplayName)
+{
+    _displayName = newDisplayName;
+    emit accountChangedDisplayName();
+}
+
 QString Account::id() const
 {
     return _id;
 }
 
-static bool isEqualExceptProtocol(const QUrl &url1, const QUrl &url2)
-{
-    return (url1.host() != url2.host() ||
-            url1.port() != url2.port() ||
-            url1.path() != url2.path());
-}
-
-bool Account::changed(AccountPtr other, bool ignoreUrlProtocol) const
-{
-    if (!other) {
-        return false;
-    }
-    bool changes = false;
-    if (ignoreUrlProtocol) {
-        changes = isEqualExceptProtocol(_url, other->_url);
-    } else {
-        changes = (_url == other->_url);
-    }
-
-    changes |= _credentials->changed(other->credentials());
-
-    return changes;
-}
-
 AbstractCredentials *Account::credentials() const
 {
     return _credentials.data();
@@ -136,39 +150,47 @@ void Account::setCredentials(AbstractCredentials *cred)
 
     // The order for these two is important! Reading the credential's
     // settings accesses the account as well as account->_credentials,
-    // so deleteLater must be used.
-    _credentials = QSharedPointer<AbstractCredentials>(cred, &QObject::deleteLater);
+    _credentials.reset(cred);
     cred->setAccount(this);
 
-    _am = QSharedPointer<QNetworkAccessManager>(_credentials->getQNAM(), &QObject::deleteLater);
+    // Note: This way the QNAM can outlive the Account and Credentials.
+    // This is necessary to avoid issues with the QNAM being deleted while
+    // processing slotHandleSslErrors().
+    _am = QSharedPointer<QNetworkAccessManager>(_credentials->createQNAM(), &QObject::deleteLater);
 
     if (jar) {
         _am->setCookieJar(jar);
     }
-    connect(_am.data(), SIGNAL(sslErrors(QNetworkReply*,QList<QSslError>)),
-            SLOT(slotHandleSslErrors(QNetworkReply*,QList<QSslError>)));
-    connect(_am.data(), SIGNAL(proxyAuthenticationRequired(QNetworkProxy,QAuthenticator*)),
-            SIGNAL(proxyAuthenticationRequired(QNetworkProxy,QAuthenticator*)));
-    connect(_credentials.data(), SIGNAL(fetched()),
-            SLOT(slotCredentialsFetched()));
-    connect(_credentials.data(), SIGNAL(asked()),
-            SLOT(slotCredentialsAsked()));
+    connect(_am.data(), SIGNAL(sslErrors(QNetworkReply *, QList<QSslError>)),
+        SLOT(slotHandleSslErrors(QNetworkReply *, QList<QSslError>)));
+    connect(_am.data(), &QNetworkAccessManager::proxyAuthenticationRequired,
+        this, &Account::proxyAuthenticationRequired);
+    connect(_credentials.data(), &AbstractCredentials::fetched,
+        this, &Account::slotCredentialsFetched);
+    connect(_credentials.data(), &AbstractCredentials::asked,
+        this, &Account::slotCredentialsAsked);
 }
 
 QUrl Account::davUrl() const
 {
-    return concatUrlPath(url(), davPath());
+    return Utility::concatUrlPath(url(), davPath());
 }
 
-QList<QNetworkCookie> Account::lastAuthCookies() const
+QUrl Account::deprecatedPrivateLinkUrl(const QByteArray &numericFileId) const
 {
-    return _am->cookieJar()->cookiesForUrl(_url);
+    return Utility::concatUrlPath(_userVisibleUrl,
+        QLatin1String("/index.php/f/") + QUrl::toPercentEncoding(QString::fromLatin1(numericFileId)));
 }
 
+/**
+ * clear all cookies. (Session cookies or not)
+ */
 void Account::clearCookieJar()
 {
-    Q_ASSERT(qobject_cast<CookieJar*>(_am->cookieJar()));
-    static_cast<CookieJar*>(_am->cookieJar())->clearSessionCookies();
+    auto jar = qobject_cast<CookieJar *>(_am->cookieJar());
+    ASSERT(jar);
+    jar->setAllCookies(QList<QNetworkCookie>());
+    emit wantsAccountSaved(this);
 }
 
 /*! This shares our official cookie jar (containing all the tasty
@@ -182,24 +204,30 @@ void Account::lendCookieJarTo(QNetworkAccessManager *guest)
     jar->setParent(oldParent); // takes it back
 }
 
+QString Account::cookieJarPath()
+{
+    ConfigFile cfg;
+    return cfg.configPath() + "/cookies" + id() + ".db";
+}
+
 void Account::resetNetworkAccessManager()
 {
     if (!_credentials || !_am) {
         return;
     }
 
-    qDebug() << "Resetting QNAM";
-    QNetworkCookieJar* jar = _am->cookieJar();
+    qCDebug(lcAccount) << "Resetting QNAM";
+    QNetworkCookieJar *jar = _am->cookieJar();
 
     // Use a QSharedPointer to allow locking the life of the QNAM on the stack.
     // Make it call deleteLater to make sure that we can return to any QNAM stack frames safely.
-    _am = QSharedPointer<QNetworkAccessManager>(_credentials->getQNAM(), &QObject::deleteLater);
+    _am = QSharedPointer<QNetworkAccessManager>(_credentials->createQNAM(), &QObject::deleteLater);
 
     _am->setCookieJar(jar); // takes ownership of the old cookie jar
-    connect(_am.data(), SIGNAL(sslErrors(QNetworkReply*,QList<QSslError>)),
-            SLOT(slotHandleSslErrors(QNetworkReply*,QList<QSslError>)));
-    connect(_am.data(), SIGNAL(proxyAuthenticationRequired(QNetworkProxy,QAuthenticator*)),
-            SIGNAL(proxyAuthenticationRequired(QNetworkProxy,QAuthenticator*)));
+    connect(_am.data(), SIGNAL(sslErrors(QNetworkReply *, QList<QSslError>)),
+        SLOT(slotHandleSslErrors(QNetworkReply *, QList<QSslError>)));
+    connect(_am.data(), &QNetworkAccessManager::proxyAuthenticationRequired,
+        this, &Account::proxyAuthenticationRequired);
 }
 
 QNetworkAccessManager *Account::networkAccessManager()
@@ -207,61 +235,34 @@ QNetworkAccessManager *Account::networkAccessManager()
     return _am.data();
 }
 
-QNetworkReply *Account::headRequest(const QString &relPath)
+QSharedPointer<QNetworkAccessManager> Account::sharedNetworkAccessManager()
 {
-    return headRequest(concatUrlPath(url(), relPath));
+    return _am;
 }
 
-QNetworkReply *Account::headRequest(const QUrl &url)
-{
-    QNetworkRequest request(url);
-#if QT_VERSION > QT_VERSION_CHECK(4, 8, 4)
-    request.setSslConfiguration(this->getOrCreateSslConfig());
-#endif
-    return _am->head(request);
-}
-
-QNetworkReply *Account::getRequest(const QString &relPath)
-{
-    return getRequest(concatUrlPath(url(), relPath));
-}
-
-QNetworkReply *Account::getRequest(const QUrl &url)
-{
-    QNetworkRequest request(url);
-#if QT_VERSION > QT_VERSION_CHECK(4, 8, 4)
-    request.setSslConfiguration(this->getOrCreateSslConfig());
-#endif
-    return _am->get(request);
-}
-
-QNetworkReply *Account::deleteRequest( const QUrl &url)
-{
-    QNetworkRequest request(url);
-#if QT_VERSION > QT_VERSION_CHECK(4, 8, 4)
-    request.setSslConfiguration(this->getOrCreateSslConfig());
-#endif
-    return _am->deleteResource(request);
-}
-
-QNetworkReply *Account::davRequest(const QByteArray &verb, const QString &relPath, QNetworkRequest req, QIODevice *data)
-{
-    return davRequest(verb, concatUrlPath(davUrl(), relPath), req, data);
-}
-
-QNetworkReply *Account::davRequest(const QByteArray &verb, const QUrl &url, QNetworkRequest req, QIODevice *data)
+QNetworkReply *Account::sendRawRequest(const QByteArray &verb, const QUrl &url, QNetworkRequest req, QIODevice *data)
 {
     req.setUrl(url);
-#if QT_VERSION > QT_VERSION_CHECK(4, 8, 4)
     req.setSslConfiguration(this->getOrCreateSslConfig());
-#endif
+    if (verb == "HEAD" && !data) {
+        return _am->head(req);
+    } else if (verb == "GET" && !data) {
+        return _am->get(req);
+    } else if (verb == "POST") {
+        return _am->post(req, data);
+    } else if (verb == "PUT") {
+        return _am->put(req, data);
+    } else if (verb == "DELETE" && !data) {
+        return _am->deleteResource(req);
+    }
     return _am->sendCustomRequest(req, verb, data);
 }
 
-void Account::setCertificate(const QByteArray certficate, const QString privateKey)
+SimpleNetworkJob *Account::sendRequest(const QByteArray &verb, const QUrl &url, QNetworkRequest req, QIODevice *data)
 {
-    _pemCertificate=certficate;
-    _pemPrivateKey=privateKey;
+    auto job = new SimpleNetworkJob(sharedFromThis(), this);
+    job->startRequest(verb, url, req, data);
+    return job;
 }
 
 void Account::setSslConfiguration(const QSslConfiguration &config)
@@ -280,37 +281,11 @@ QSslConfiguration Account::getOrCreateSslConfig()
     // if setting the client certificate fails, you will probably get an error similar to this:
     //  "An internal error number 1060 happened. SSL handshake failed, client certificate was requested: SSL error: sslv3 alert handshake failure"
     QSslConfiguration sslConfig = QSslConfiguration::defaultConfiguration();
-    QSslCertificate sslClientCertificate;
-    
-    ConfigFile cfgFile;
-    if(!cfgFile.certificatePath().isEmpty() && !cfgFile.certificatePasswd().isEmpty()) {
-        resultP12ToPem certif = p12ToPem(cfgFile.certificatePath().toStdString(), cfgFile.certificatePasswd().toStdString());
-        QString s = QString::fromStdString(certif.Certificate);
-        QByteArray ba = s.toLocal8Bit();
-        this->setCertificate(ba, QString::fromStdString(certif.PrivateKey));
-    }
-    if((!_pemCertificate.isEmpty())&&(!_pemPrivateKey.isEmpty())) {
-        // Read certificates
-        QList<QSslCertificate> sslCertificateList = QSslCertificate::fromData(_pemCertificate, QSsl::Pem);
-        if(sslCertificateList.length() != 0) {
-            sslClientCertificate = sslCertificateList.takeAt(0);
-        }
-        // Read key from file
-        QSslKey privateKey(_pemPrivateKey.toLocal8Bit(), QSsl::Rsa, QSsl::Pem, QSsl::PrivateKey , "");
 
-        // SSL configuration
-        sslConfig.setCaCertificates(QSslSocket::systemCaCertificates());
-        sslConfig.setLocalCertificate(sslClientCertificate);
-        sslConfig.setPrivateKey(privateKey);
-        qDebug() << "Added SSL client certificate to the query";
-    }
-
-#if QT_VERSION > QT_VERSION_CHECK(5, 2, 0)
     // Try hard to re-use session for different requests
     sslConfig.setSslOption(QSsl::SslOptionDisableSessionTickets, false);
     sslConfig.setSslOption(QSsl::SslOptionDisableSessionSharing, false);
     sslConfig.setSslOption(QSsl::SslOptionDisableSessionPersistence, false);
-#endif
 
     return sslConfig;
 }
@@ -338,50 +313,19 @@ void Account::setSslErrorHandler(AbstractSslErrorHandler *handler)
 void Account::setUrl(const QUrl &url)
 {
     _url = url;
+    _userVisibleUrl = url;
 }
 
-QUrl Account::concatUrlPath(const QUrl &url, const QString &concatPath,
-                            const QList< QPair<QString, QString> > &queryItems)
+void Account::setUserVisibleHost(const QString &host)
 {
-    QString path = url.path();
-    if (! concatPath.isEmpty()) {
-        // avoid '//'
-        if (path.endsWith('/') && concatPath.startsWith('/')) {
-            path.chop(1);
-        } // avoid missing '/'
-        else if (!path.endsWith('/') && !concatPath.startsWith('/')) {
-            path += QLatin1Char('/');
-        }
-        path += concatPath; // put the complete path together
-    }
-
-    QUrl tmpUrl = url;
-    tmpUrl.setPath(path);
-    if( queryItems.size() > 0 ) {
-        tmpUrl.setQueryItems(queryItems);
-    }
-    return tmpUrl;
-}
-
-QString Account::_configFileName;
-
-std::unique_ptr<QSettings> Account::settingsWithGroup(const QString& group, QObject *parent)
-{
-    if (_configFileName.isEmpty()) {
-        // cache file name
-        ConfigFile cfg;
-        _configFileName = cfg.configFile();
-    }
-    std::unique_ptr<QSettings> settings(new QSettings(_configFileName, QSettings::IniFormat, parent));
-    settings->beginGroup(group);
-    return settings;
+    _userVisibleUrl.setHost(host);
 }
 
 QVariant Account::credentialSetting(const QString &key) const
 {
     if (_credentials) {
         QString prefix = _credentials->authType();
-        QString value = _settingsMap.value(prefix+"_"+key).toString();
+        QString value = _settingsMap.value(prefix + "_" + key).toString();
         if (value.isEmpty()) {
             value = _settingsMap.value(key).toString();
         }
@@ -394,18 +338,19 @@ void Account::setCredentialSetting(const QString &key, const QVariant &value)
 {
     if (_credentials) {
         QString prefix = _credentials->authType();
-        _settingsMap.insert(prefix+"_"+key, value);
+        _settingsMap.insert(prefix + "_" + key, value);
     }
 }
 
-void Account::slotHandleSslErrors(QNetworkReply *reply , QList<QSslError> errors)
+void Account::slotHandleSslErrors(QNetworkReply *reply, QList<QSslError> errors)
 {
     NetworkJobTimeoutPauser pauser(reply);
     QString out;
     QDebug(&out) << "SSL-Errors happened for url " << reply->url().toString();
-    foreach(const QSslError &error, errors) {
+    foreach (const QSslError &error, errors) {
         QDebug(&out) << "\tError in " << error.certificate() << ":"
-                     << error.errorString() << "("<< error.error() << ")" << "\n";
+                     << error.errorString() << "(" << error.error() << ")"
+                     << "\n";
     }
 
     bool allPreviouslyRejected = true;
@@ -416,14 +361,14 @@ void Account::slotHandleSslErrors(QNetworkReply *reply , QList<QSslError> errors
     }
 
     // If all certs have previously been rejected by the user, don't ask again.
-    if( allPreviouslyRejected ) {
-        qDebug() << out << "Certs not trusted by user decision, returning.";
+    if (allPreviouslyRejected) {
+        qCInfo(lcAccount) << out << "Certs not trusted by user decision, returning.";
         return;
     }
 
     QList<QSslCertificate> approvedCerts;
-    if (_sslErrorHandler.isNull() ) {
-        qDebug() << out << Q_FUNC_INFO << "called without valid SSL error handler for account" << url();
+    if (_sslErrorHandler.isNull()) {
+        qCWarning(lcAccount) << out << "called without valid SSL error handler for account" << url();
         return;
     }
 
@@ -432,30 +377,34 @@ void Account::slotHandleSslErrors(QNetworkReply *reply , QList<QSslError> errors
     // Keep a ref here on our stackframe to make sure that it doesn't get deleted before
     // handleErrors returns.
     QSharedPointer<QNetworkAccessManager> qnamLock = _am;
+    QPointer<QObject> guard = reply;
 
     if (_sslErrorHandler->handleErrors(errors, reply->sslConfiguration(), &approvedCerts, sharedFromThis())) {
+        if (!guard)
+            return;
+
         QSslSocket::addDefaultCaCertificates(approvedCerts);
         addApprovedCerts(approvedCerts);
         emit wantsAccountSaved(this);
         // all ssl certs are known and accepted. We can ignore the problems right away.
-//         qDebug() << out << "Certs are known and trusted! This is not an actual error.";
+        qCInfo(lcAccount) << out << "Certs are known and trusted! This is not an actual error.";
 
         // Warning: Do *not* use ignoreSslErrors() (without args) here:
         // it permanently ignores all SSL errors for this host, even
         // certificate changes.
         reply->ignoreSslErrors(errors);
     } else {
+        if (!guard)
+            return;
+
         // Mark all involved certificates as rejected, so we don't ask the user again.
         foreach (const QSslError &error, errors) {
             if (!_rejectedCertificates.contains(error.certificate())) {
                 _rejectedCertificates.append(error.certificate());
             }
         }
-        // if during normal operation, a new certificate was MITM'ed, and the user does not
-        // ACK it, the running request must be aborted and the QNAM must be reset, to not
-        // treat the new cert as granted. See bug #3283
-        reply->abort();
-        resetNetworkAccessManager();
+
+        // Not calling ignoreSslErrors will make the SSL handshake fail.
         return;
     }
 }
@@ -475,14 +424,9 @@ void Account::handleInvalidCredentials()
     emit invalidCredentials();
 }
 
-bool Account::wasMigrated()
+void Account::clearQNAMCache()
 {
-    return _wasMigrated;
-}
-
-void Account::setMigrated(bool mig)
-{
-    _wasMigrated = mig;
+    _am->clearAccessCache();
 }
 
 const Capabilities &Account::capabilities() const
@@ -504,9 +448,14 @@ int Account::serverVersionInt() const
 {
     // FIXME: Use Qt 5.5 QVersionNumber
     auto components = serverVersion().split('.');
-    return  (components.value(0).toInt() << 16)
-                   + (components.value(1).toInt() << 8)
-            + components.value(2).toInt();
+    return makeServerVersion(components.value(0).toInt(),
+        components.value(1).toInt(),
+        components.value(2).toInt());
+}
+
+int Account::makeServerVersion(int majorVersion, int minorVersion, int patchVersion)
+{
+    return (majorVersion << 16) + (minorVersion << 8) + patchVersion;
 }
 
 bool Account::serverVersionUnsupported() const
@@ -515,10 +464,10 @@ bool Account::serverVersionUnsupported() const
         // not detected yet, assume it is fine.
         return false;
     }
-    return serverVersionInt() < 0x070000;
+    return serverVersionInt() < makeServerVersion(7, 0, 0);
 }
 
-void Account::setServerVersion(const QString& version)
+void Account::setServerVersion(const QString &version)
 {
     if (version == _serverVersion) {
         return;
@@ -531,18 +480,17 @@ void Account::setServerVersion(const QString& version)
 
 bool Account::rootEtagChangesNotOnlySubFolderEtags()
 {
-    return (serverVersionInt() >= 0x080100);
+    return (serverVersionInt() >= makeServerVersion(8, 1, 0));
 }
 
 void Account::setNonShib(bool nonShib)
 {
-    if( nonShib ) {
+    if (nonShib) {
         _davPath = Theme::instance()->webDavPathNonShib();
     } else {
         _davPath = Theme::instance()->webDavPath();
-    } 
+    }
 }
 
 
-
 } // namespace OCC
