@@ -3,7 +3,8 @@
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; version 2 of the License.
+ * the Free Software Foundation; either version 2 of the License, or
+ * (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful, but
  * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
@@ -20,7 +21,8 @@
 #include <QUrl>
 
 #include <csync.h>
-#include "utility.h"
+#include "common/utility.h"
+#include "owncloudlib.h"
 
 namespace OCC {
 
@@ -43,23 +45,23 @@ public slots:
     void setupQtProxyFromConfig();
 
 private:
-    const char* proxyTypeToCStr(QNetworkProxy::ProxyType type);
+    const char *proxyTypeToCStr(QNetworkProxy::ProxyType type);
 };
 
-class SystemProxyRunnable : public QObject, public QRunnable {
+class SystemProxyRunnable : public QObject, public QRunnable
+{
     Q_OBJECT
 public:
     SystemProxyRunnable(const QUrl &url);
     void run();
 signals:
     void systemProxyLookedUp(const QNetworkProxy &url);
+
 private:
     QUrl _url;
 };
 
-QString printQNetworkProxy(const QNetworkProxy &proxy);
-
-
+OWNCLOUDSYNC_EXPORT QString printQNetworkProxy(const QNetworkProxy &proxy);
 }
 
 #endif // CLIENTPROXY_H
