@@ -16,7 +16,8 @@
 
 #include "configfile.h"
 #include "theme.h"
-#include "utility.h"
+#include "common/utility.h"
+#include "common/asserts.h"
 
 #include "creds/abstractcredentials.h"
 
@@ -30,8 +31,8 @@
 #include <QDir>
 #include <QFile>
 #include <QFileInfo>
+#include <QLoggingCategory>
 #include <QSettings>
-#include <QDebug>
 #include <QNetworkProxy>
 
 #define DEFAULT_REMOTE_POLL_INTERVAL 30000 // default remote poll time in milliseconds
@@ -39,10 +40,14 @@
 
 namespace OCC {
 
+Q_LOGGING_CATEGORY(lcConfigFile, "sync.configfile", QtInfoMsg)
+
 //static const char caCertsKeyC[] = "CaCertificates"; only used from account.cpp
 static const char remotePollIntervalC[] = "remotePollInterval";
 static const char forceSyncIntervalC[] = "forceSyncInterval";
+static const char notificationRefreshIntervalC[] = "notificationRefreshInterval";
 static const char monoIconsC[] = "monoIcons";
+static const char promptDeleteC[] = "promptDeleteAllFiles";
 static const char crashReporterC[] = "crashReporter";
 static const char optionalDesktopNoficationsC[] = "optionalDesktopNotifications";
 static const char skipUpdateCheckC[] = "skipUpdateCheck";
@@ -50,7 +55,9 @@ static const char updateCheckIntervalC[] = "updateCheckInterval";
 static const char geometryC[] = "geometry";
 static const char timeoutC[] = "timeout";
 static const char chunkSizeC[] = "chunkSize";
-static const char transmissionChecksumC[] = "transmissionChecksum";
+static const char minChunkSizeC[] = "minChunkSize";
+static const char maxChunkSizeC[] = "maxChunkSize";
+static const char targetChunkUploadDurationC[] = "targetChunkUploadDuration";
 
 static const char proxyHostC[] = "Proxy/host";
 static const char proxyTypeC[] = "Proxy/type";
@@ -59,25 +66,26 @@ static const char proxyUserC[] = "Proxy/user";
 static const char proxyPassC[] = "Proxy/pass";
 static const char proxyNeedsAuthC[] = "Proxy/needsAuth";
 
-static const char useUploadLimitC[]   = "BWLimit/useUploadLimit";
+static const char useUploadLimitC[] = "BWLimit/useUploadLimit";
 static const char useDownloadLimitC[] = "BWLimit/useDownloadLimit";
-static const char uploadLimitC[]      = "BWLimit/uploadLimit";
-static const char downloadLimitC[]    = "BWLimit/downloadLimit";
+static const char uploadLimitC[] = "BWLimit/uploadLimit";
+static const char downloadLimitC[] = "BWLimit/downloadLimit";
 
 static const char newBigFolderSizeLimitC[] = "newBigFolderSizeLimit";
 static const char useNewBigFolderSizeLimitC[] = "useNewBigFolderSizeLimit";
+static const char confirmExternalStorageC[] = "confirmExternalStorage";
 
 static const char maxLogLinesC[] = "Logging/maxLogLines";
 
 const char certPath[] = "http_certificatePath";
 const char certPasswd[] = "http_certificatePasswd";
 QString ConfigFile::_confDir = QString::null;
-bool    ConfigFile::_askedUser = false;
+bool ConfigFile::_askedUser = false;
 
 ConfigFile::ConfigFile()
 {
     // QDesktopServices uses the application name to create a config path
-    qApp->setApplicationName( Theme::instance()->appNameGUI() );
+    qApp->setApplicationName(Theme::instance()->appNameGUI());
 
     QSettings::setDefaultFormat(QSettings::IniFormat);
 
@@ -85,25 +93,24 @@ ConfigFile::ConfigFile()
 
 
     QSettings settings(config, QSettings::IniFormat);
-    settings.beginGroup( defaultConnection() );
-
-    // qDebug() << Q_FUNC_INFO << "Loading config: " << config << " (URL is " << settings.value("url").toString() << ")";
+    settings.beginGroup(defaultConnection());
 }
 
 bool ConfigFile::setConfDir(const QString &value)
 {
     QString dirPath = value;
-    if( dirPath.isEmpty() ) return false;
+    if (dirPath.isEmpty())
+        return false;
 
     QFileInfo fi(dirPath);
-    if ( !fi.exists()  ) {
+    if (!fi.exists()) {
         QDir().mkpath(dirPath);
         fi.setFile(dirPath);
     }
-    if( fi.exists() && fi.isDir() ) {
+    if (fi.exists() && fi.isDir()) {
         dirPath = fi.absoluteFilePath();
-        qDebug() << "** Using custom config dir " << dirPath;
-        _confDir=dirPath;
+        qCInfo(lcConfigFile) << "Using custom config dir " << dirPath;
+        _confDir = dirPath;
         return true;
     }
     return false;
@@ -124,21 +131,25 @@ int ConfigFile::timeout() const
 quint64 ConfigFile::chunkSize() const
 {
     QSettings settings(configFile(), QSettings::IniFormat);
-    return settings.value(QLatin1String(chunkSizeC), 5*1024*1024).toLongLong(); // default to 5 MiB
+    return settings.value(QLatin1String(chunkSizeC), 10 * 1000 * 1000).toLongLong(); // default to 10 MB
 }
 
-QString ConfigFile::transmissionChecksum() const
+quint64 ConfigFile::maxChunkSize() const
 {
     QSettings settings(configFile(), QSettings::IniFormat);
+    return settings.value(QLatin1String(maxChunkSizeC), 100 * 1000 * 1000).toLongLong(); // default to 100 MB
+}
 
-    QString checksum = settings.value(QLatin1String(transmissionChecksumC), QString()).toString();
+quint64 ConfigFile::minChunkSize() const
+{
+    QSettings settings(configFile(), QSettings::IniFormat);
+    return settings.value(QLatin1String(minChunkSizeC), 1000 * 1000).toLongLong(); // default to 1 MB
+}
 
-    if( checksum.isEmpty() ) {
-        // if the config file setting is empty, maybe the Branding requires it.
-        checksum = Theme::instance()->transmissionChecksum();
-    }
-
-    return checksum;
+quint64 ConfigFile::targetChunkUploadDuration() const
+{
+    QSettings settings(configFile(), QSettings::IniFormat);
+    return settings.value(QLatin1String(targetChunkUploadDurationC), 60 * 1000).toLongLong(); // default to 1 minute
 }
 
 void ConfigFile::setOptionalDesktopNotifications(bool show)
@@ -151,7 +162,7 @@ void ConfigFile::setOptionalDesktopNotifications(bool show)
 void ConfigFile::saveGeometry(QWidget *w)
 {
 #ifndef TOKEN_AUTH_ONLY
-    Q_ASSERT(!w->objectName().isNull());
+    ASSERT(!w->objectName().isNull());
     QSettings settings(configFile(), QSettings::IniFormat);
     settings.beginGroup(w->objectName());
     settings.setValue(QLatin1String(geometryC), w->saveGeometry());
@@ -169,8 +180,9 @@ void ConfigFile::restoreGeometry(QWidget *w)
 void ConfigFile::saveGeometryHeader(QHeaderView *header)
 {
 #ifndef TOKEN_AUTH_ONLY
-    if(!header) return;
-    Q_ASSERT(!header->objectName().isEmpty());
+    if (!header)
+        return;
+    ASSERT(!header->objectName().isEmpty());
 
     QSettings settings(configFile(), QSettings::IniFormat);
     settings.beginGroup(header->objectName());
@@ -182,8 +194,9 @@ void ConfigFile::saveGeometryHeader(QHeaderView *header)
 void ConfigFile::restoreGeometryHeader(QHeaderView *header)
 {
 #ifndef TOKEN_AUTH_ONLY
-    if(!header) return;
-    Q_ASSERT(!header->objectName().isNull());
+    if (!header)
+        return;
+    ASSERT(!header->objectName().isNull());
 
     QSettings settings(configFile(), QSettings::IniFormat);
     settings.beginGroup(header->objectName());
@@ -191,21 +204,21 @@ void ConfigFile::restoreGeometryHeader(QHeaderView *header)
 #endif
 }
 
-QVariant ConfigFile::getPolicySetting(const QString &setting, const QVariant& defaultValue) const
+QVariant ConfigFile::getPolicySetting(const QString &setting, const QVariant &defaultValue) const
 {
     if (Utility::isWindows()) {
         // check for policies first and return immediately if a value is found.
         QSettings userPolicy(QString::fromLatin1("HKEY_CURRENT_USER\\Software\\Policies\\%1\\%2")
-                             .arg(APPLICATION_VENDOR).arg(Theme::instance()->appName()),
-                             QSettings::NativeFormat);
-        if(userPolicy.contains(setting)) {
+                                 .arg(APPLICATION_VENDOR, Theme::instance()->appName()),
+            QSettings::NativeFormat);
+        if (userPolicy.contains(setting)) {
             return userPolicy.value(setting);
         }
 
         QSettings machinePolicy(QString::fromLatin1("HKEY_LOCAL_MACHINE\\Software\\Policies\\%1\\%2")
-                                .arg(APPLICATION_VENDOR).arg(APPLICATION_NAME),
-                                QSettings::NativeFormat);
-        if(machinePolicy.contains(setting)) {
+                                    .arg(APPLICATION_VENDOR, APPLICATION_NAME),
+            QSettings::NativeFormat);
+        if (machinePolicy.contains(setting)) {
             return machinePolicy.value(setting);
         }
     }
@@ -214,23 +227,24 @@ QVariant ConfigFile::getPolicySetting(const QString &setting, const QVariant& de
 
 QString ConfigFile::configPath() const
 {
-    #ifndef TOKEN_AUTH_ONLY
-    if( _confDir.isEmpty() ) {
+#ifndef TOKEN_AUTH_ONLY
+    if (_confDir.isEmpty()) {
         //  Qt 5's QStandardPaths::writableLocation gives us wrong results (without /data/),
         //  so we'll have to use the deprecated version for now
         _confDir = QDesktopServices::storageLocation(QDesktopServices::DataLocation);
     }
-    #endif
+#endif
     QString dir = _confDir;
 
-    if( !dir.endsWith(QLatin1Char('/')) ) dir.append(QLatin1Char('/'));
+    if (!dir.endsWith(QLatin1Char('/')))
+        dir.append(QLatin1Char('/'));
     return dir;
 }
 
 QString ConfigFile::configPathWithAppName() const
 {
     //HACK
-    return QFileInfo( configFile() ).dir().absolutePath().append("/");
+    return QFileInfo(configFile()).dir().absolutePath().append("/");
 }
 
 static const QLatin1String exclFile("sync-exclude.lst");
@@ -242,37 +256,37 @@ QString ConfigFile::excludeFile(Scope scope) const
     // directories.
     QFileInfo fi;
 
-    if (scope != SystemScope) {
-        QFileInfo fi;
-        fi.setFile( configPath(), exclFile );
+    switch (scope) {
+    case UserScope:
+        fi.setFile(configPath(), exclFile);
 
-        if( ! fi.isReadable() ) {
-            fi.setFile( configPath(), QLatin1String("exclude.lst") );
+        if (!fi.isReadable()) {
+            fi.setFile(configPath(), QLatin1String("exclude.lst"));
         }
-        if( ! fi.isReadable() ) {
-            fi.setFile( configPath(), exclFile );
+        if (!fi.isReadable()) {
+            fi.setFile(configPath(), exclFile);
         }
         return fi.absoluteFilePath();
-    } else if (scope != UserScope) {
+    case SystemScope:
         return ConfigFile::excludeFileFromSystem();
-    } else {
-        Q_ASSERT(false);
-        return QString(); // unreachable
     }
+
+    ASSERT(false);
+    return QString();
 }
 
 QString ConfigFile::excludeFileFromSystem()
 {
     QFileInfo fi;
 #ifdef Q_OS_WIN
-    fi.setFile( QCoreApplication::applicationDirPath(), exclFile );
+    fi.setFile(QCoreApplication::applicationDirPath(), exclFile);
 #endif
 #ifdef Q_OS_UNIX
-    fi.setFile( QString( SYSCONFDIR "/%1").arg(Theme::instance()->appName()), exclFile );
-    if ( ! fi.exists() ) {
+    fi.setFile(QString(SYSCONFDIR "/" + Theme::instance()->appName()), exclFile);
+    if (!fi.exists()) {
         // Prefer to return the preferred path! Only use the fallback location
         // if the other path does not exist and the fallback is valid.
-        QFileInfo nextToBinary( QCoreApplication::applicationDirPath(), exclFile );
+        QFileInfo nextToBinary(QCoreApplication::applicationDirPath(), exclFile);
         if (nextToBinary.exists()) {
             fi = nextToBinary;
         }
@@ -280,10 +294,10 @@ QString ConfigFile::excludeFileFromSystem()
 #endif
 #ifdef Q_OS_MAC
     // exec path is inside the bundle
-    fi.setFile( QCoreApplication::applicationDirPath(),
-                QLatin1String("../Resources/") + exclFile );
+    fi.setFile(QCoreApplication::applicationDirPath(),
+        QLatin1String("../Resources/") + exclFile);
 #endif
-    
+
     return fi.absoluteFilePath();
 }
 
@@ -294,7 +308,7 @@ QString ConfigFile::configFile() const
 
 bool ConfigFile::exists()
 {
-    QFile file( configFile() );
+    QFile file(configFile());
     return file.exists();
 }
 
@@ -303,7 +317,7 @@ QString ConfigFile::defaultConnection() const
     return Theme::instance()->appName();
 }
 
-void ConfigFile::storeData(const QString& group, const QString& key, const QVariant& value)
+void ConfigFile::storeData(const QString &group, const QString &key, const QVariant &value)
 {
     const QString con(group.isEmpty() ? defaultConnection() : group);
     QSettings settings(configFile(), QSettings::IniFormat);
@@ -313,7 +327,7 @@ void ConfigFile::storeData(const QString& group, const QString& key, const QVari
     settings.sync();
 }
 
-QVariant ConfigFile::retrieveData(const QString& group, const QString& key) const
+QVariant ConfigFile::retrieveData(const QString &group, const QString &key) const
 {
     const QString con(group.isEmpty() ? defaultConnection() : group);
     QSettings settings(configFile(), QSettings::IniFormat);
@@ -322,7 +336,7 @@ QVariant ConfigFile::retrieveData(const QString& group, const QString& key) cons
     return settings.value(key);
 }
 
-void ConfigFile::removeData(const QString& group, const QString& key)
+void ConfigFile::removeData(const QString &group, const QString &key)
 {
     const QString con(group.isEmpty() ? defaultConnection() : group);
     QSettings settings(configFile(), QSettings::IniFormat);
@@ -331,7 +345,7 @@ void ConfigFile::removeData(const QString& group, const QString& key)
     settings.remove(key);
 }
 
-bool ConfigFile::dataExists(const QString& group, const QString& key) const
+bool ConfigFile::dataExists(const QString &group, const QString &key) const
 {
     const QString con(group.isEmpty() ? defaultConnection() : group);
     QSettings settings(configFile(), QSettings::IniFormat);
@@ -340,77 +354,99 @@ bool ConfigFile::dataExists(const QString& group, const QString& key) const
     return settings.contains(key);
 }
 
-int ConfigFile::remotePollInterval( const QString& connection ) const
+int ConfigFile::remotePollInterval(const QString &connection) const
 {
-  QString con( connection );
-  if( connection.isEmpty() ) con = defaultConnection();
+    QString con(connection);
+    if (connection.isEmpty())
+        con = defaultConnection();
 
-  QSettings settings(configFile(), QSettings::IniFormat);
-  settings.beginGroup( con );
+    QSettings settings(configFile(), QSettings::IniFormat);
+    settings.beginGroup(con);
 
-  int remoteInterval = settings.value( QLatin1String(remotePollIntervalC), DEFAULT_REMOTE_POLL_INTERVAL ).toInt();
-  if( remoteInterval < 5000) {
-    qDebug() << "Remote Interval is less than 5 seconds, reverting to" << DEFAULT_REMOTE_POLL_INTERVAL;
-    remoteInterval = DEFAULT_REMOTE_POLL_INTERVAL;
-  }
-  return remoteInterval;
+    int remoteInterval = settings.value(QLatin1String(remotePollIntervalC), DEFAULT_REMOTE_POLL_INTERVAL).toInt();
+    if (remoteInterval < 5000) {
+        qCWarning(lcConfigFile) << "Remote Interval is less than 5 seconds, reverting to" << DEFAULT_REMOTE_POLL_INTERVAL;
+        remoteInterval = DEFAULT_REMOTE_POLL_INTERVAL;
+    }
+    return remoteInterval;
 }
 
-void ConfigFile::setRemotePollInterval(int interval, const QString &connection )
+void ConfigFile::setRemotePollInterval(int interval, const QString &connection)
 {
-    QString con( connection );
-    if( connection.isEmpty() ) con = defaultConnection();
+    QString con(connection);
+    if (connection.isEmpty())
+        con = defaultConnection();
 
-    if( interval < 5000 ) {
-        qDebug() << "Remote Poll interval of " << interval << " is below five seconds.";
+    if (interval < 5000) {
+        qCWarning(lcConfigFile) << "Remote Poll interval of " << interval << " is below five seconds.";
         return;
     }
     QSettings settings(configFile(), QSettings::IniFormat);
-    settings.beginGroup( con );
-    settings.setValue(QLatin1String(remotePollIntervalC), interval );
+    settings.beginGroup(con);
+    settings.setValue(QLatin1String(remotePollIntervalC), interval);
     settings.sync();
 }
 
-quint64 ConfigFile::forceSyncInterval(const QString& connection) const
+quint64 ConfigFile::forceSyncInterval(const QString &connection) const
 {
     uint pollInterval = remotePollInterval(connection);
 
-    QString con( connection );
-    if( connection.isEmpty() ) con = defaultConnection();
+    QString con(connection);
+    if (connection.isEmpty())
+        con = defaultConnection();
     QSettings settings(configFile(), QSettings::IniFormat);
-    settings.beginGroup( con );
+    settings.beginGroup(con);
 
     quint64 defaultInterval = 2 * 60 * 60 * 1000ull; // 2h
-    quint64 interval = settings.value( QLatin1String(forceSyncIntervalC), defaultInterval ).toULongLong();
-    if( interval < pollInterval) {
-        qDebug() << "Force sync interval is less than the remote poll inteval, reverting to" << pollInterval;
+    quint64 interval = settings.value(QLatin1String(forceSyncIntervalC), defaultInterval).toULongLong();
+    if (interval < pollInterval) {
+        qCWarning(lcConfigFile) << "Force sync interval is less than the remote poll inteval, reverting to" << pollInterval;
         interval = pollInterval;
     }
     return interval;
 }
 
-int ConfigFile::updateCheckInterval( const QString& connection ) const
+quint64 ConfigFile::notificationRefreshInterval(const QString &connection) const
 {
-    QString con( connection );
-    if( connection.isEmpty() ) con = defaultConnection();
+    QString con(connection);
+    if (connection.isEmpty())
+        con = defaultConnection();
     QSettings settings(configFile(), QSettings::IniFormat);
-    settings.beginGroup( con );
+    settings.beginGroup(con);
 
-    int defaultInterval = 1000*60*60*10; // ten hours
-    int interval = settings.value( QLatin1String(updateCheckIntervalC), defaultInterval ).toInt();
+    quint64 defaultInterval = 5 * 60 * 1000ull; // 5 minutes
+    quint64 interval = settings.value(QLatin1String(notificationRefreshIntervalC), defaultInterval).toULongLong();
+    if (interval < 60 * 1000ull) {
+        qCWarning(lcConfigFile) << "Notification refresh interval smaller than one minute, setting to one minute";
+        interval = 60 * 1000ull;
+    }
+    return interval;
+}
 
-    int minInterval = 1000*60*5;
-    if( interval < minInterval) {
-        qDebug() << "Update check interval less than five minutes, setting " << minInterval;
+int ConfigFile::updateCheckInterval(const QString &connection) const
+{
+    QString con(connection);
+    if (connection.isEmpty())
+        con = defaultConnection();
+    QSettings settings(configFile(), QSettings::IniFormat);
+    settings.beginGroup(con);
+
+    int defaultInterval = 1000 * 60 * 60 * 10; // ten hours
+    int interval = settings.value(QLatin1String(updateCheckIntervalC), defaultInterval).toInt();
+
+    int minInterval = 1000 * 60 * 5;
+    if (interval < minInterval) {
+        qCWarning(lcConfigFile) << "Update check interval less than five minutes, setting " << minInterval;
         interval = minInterval;
     }
     return interval;
 }
 
-bool ConfigFile::skipUpdateCheck( const QString& connection ) const
+bool ConfigFile::skipUpdateCheck(const QString &connection) const
 {
-    QString con( connection );
-    if( connection.isEmpty() ) con = defaultConnection();
+    QString con(connection);
+    if (connection.isEmpty())
+        con = defaultConnection();
 
     QVariant fallback = getValue(QLatin1String(skipUpdateCheckC), con, false);
     fallback = getValue(QLatin1String(skipUpdateCheckC), QString(), fallback);
@@ -419,26 +455,26 @@ bool ConfigFile::skipUpdateCheck( const QString& connection ) const
     return value.toBool();
 }
 
-void ConfigFile::setSkipUpdateCheck( bool skip, const QString& connection )
+void ConfigFile::setSkipUpdateCheck(bool skip, const QString &connection)
 {
-    QString con( connection );
-    if( connection.isEmpty() ) con = defaultConnection();
+    QString con(connection);
+    if (connection.isEmpty())
+        con = defaultConnection();
 
     QSettings settings(configFile(), QSettings::IniFormat);
-    settings.beginGroup( con );
+    settings.beginGroup(con);
 
-    settings.setValue( QLatin1String(skipUpdateCheckC), QVariant(skip) );
+    settings.setValue(QLatin1String(skipUpdateCheckC), QVariant(skip));
     settings.sync();
-
 }
 
 int ConfigFile::maxLogLines() const
 {
     QSettings settings(configFile(), QSettings::IniFormat);
-    return settings.value( QLatin1String(maxLogLinesC), DEFAULT_MAX_LOG_LINES ).toInt();
+    return settings.value(QLatin1String(maxLogLinesC), DEFAULT_MAX_LOG_LINES).toInt();
 }
 
-void ConfigFile::setMaxLogLines( int lines )
+void ConfigFile::setMaxLogLines(int lines)
 {
     QSettings settings(configFile(), QSettings::IniFormat);
     settings.setValue(QLatin1String(maxLogLinesC), lines);
@@ -446,17 +482,16 @@ void ConfigFile::setMaxLogLines( int lines )
 }
 
 void ConfigFile::setProxyType(int proxyType,
-                  const QString& host,
-                  int port, bool needsAuth,
-                  const QString& user,
-                  const QString& pass)
+    const QString &host,
+    int port, bool needsAuth,
+    const QString &user,
+    const QString &pass)
 {
     QSettings settings(configFile(), QSettings::IniFormat);
 
     settings.setValue(QLatin1String(proxyTypeC), proxyType);
 
-    if (proxyType == QNetworkProxy::HttpProxy ||
-        proxyType == QNetworkProxy::Socks5Proxy) {
+    if (proxyType == QNetworkProxy::HttpProxy || proxyType == QNetworkProxy::Socks5Proxy) {
         settings.setValue(QLatin1String(proxyHostC), host);
         settings.setValue(QLatin1String(proxyPortC), port);
         settings.setValue(QLatin1String(proxyNeedsAuthC), needsAuth);
@@ -466,26 +501,26 @@ void ConfigFile::setProxyType(int proxyType,
     settings.sync();
 }
 
-QVariant ConfigFile::getValue(const QString& param, const QString& group,
-                                    const QVariant& defaultValue) const
+QVariant ConfigFile::getValue(const QString &param, const QString &group,
+    const QVariant &defaultValue) const
 {
     QVariant systemSetting;
     if (Utility::isMac()) {
-            QSettings systemSettings(QLatin1String("/Library/Preferences/" APPLICATION_REV_DOMAIN ".plist"), QSettings::NativeFormat);
-            if (!group.isEmpty()) {
-                systemSettings.beginGroup(group);
-            }
-            systemSetting = systemSettings.value(param, defaultValue);
+        QSettings systemSettings(QLatin1String("/Library/Preferences/" APPLICATION_REV_DOMAIN ".plist"), QSettings::NativeFormat);
+        if (!group.isEmpty()) {
+            systemSettings.beginGroup(group);
+        }
+        systemSetting = systemSettings.value(param, defaultValue);
     } else if (Utility::isUnix()) {
-        QSettings systemSettings(QString( SYSCONFDIR "/%1/%1.conf").arg(Theme::instance()->appName()), QSettings::NativeFormat);
+        QSettings systemSettings(QString(SYSCONFDIR "/%1/%1.conf").arg(Theme::instance()->appName()), QSettings::NativeFormat);
         if (!group.isEmpty()) {
             systemSettings.beginGroup(group);
         }
         systemSetting = systemSettings.value(param, defaultValue);
     } else { // Windows
         QSettings systemSettings(QString::fromLatin1("HKEY_LOCAL_MACHINE\\Software\\%1\\%2")
-                                .arg(APPLICATION_VENDOR).arg(Theme::instance()->appName()),
-                                QSettings::NativeFormat);
+                                     .arg(APPLICATION_VENDOR, Theme::instance()->appName()),
+            QSettings::NativeFormat);
         if (!group.isEmpty()) {
             systemSettings.beginGroup(group);
         }
@@ -493,12 +528,13 @@ QVariant ConfigFile::getValue(const QString& param, const QString& group,
     }
 
     QSettings settings(configFile(), QSettings::IniFormat);
-    if (!group.isEmpty()) settings.beginGroup(group);
+    if (!group.isEmpty())
+        settings.beginGroup(group);
 
     return settings.value(param, systemSetting);
 }
 
-void ConfigFile::setValue(const QString& key, const QVariant &value)
+void ConfigFile::setValue(const QString &key, const QVariant &value)
 {
     QSettings settings(configFile(), QSettings::IniFormat);
 
@@ -507,6 +543,9 @@ void ConfigFile::setValue(const QString& key, const QVariant &value)
 
 int ConfigFile::proxyType() const
 {
+    if (Theme::instance()->forceSystemNetworkProxy()) {
+        return QNetworkProxy::DefaultProxy;
+    }
     return getValue(QLatin1String(proxyTypeC)).toInt();
 }
 
@@ -590,10 +629,37 @@ void ConfigFile::setNewBigFolderSizeLimit(bool isChecked, quint64 mbytes)
     setValue(useNewBigFolderSizeLimitC, isChecked);
 }
 
+bool ConfigFile::confirmExternalStorage() const
+{
+    return getValue(confirmExternalStorageC, QString(), true).toBool();
+}
+
+void ConfigFile::setConfirmExternalStorage(bool isChecked)
+{
+    setValue(confirmExternalStorageC, isChecked);
+}
+
+bool ConfigFile::promptDeleteFiles() const
+{
+    QSettings settings(configFile(), QSettings::IniFormat);
+    return settings.value(QLatin1String(promptDeleteC), true).toBool();
+}
+
+void ConfigFile::setPromptDeleteFiles(bool promptDeleteFiles)
+{
+    QSettings settings(configFile(), QSettings::IniFormat);
+    settings.setValue(QLatin1String(promptDeleteC), promptDeleteFiles);
+}
+
 bool ConfigFile::monoIcons() const
 {
     QSettings settings(configFile(), QSettings::IniFormat);
-    return settings.value(QLatin1String(monoIconsC), false).toBool();
+    bool monoDefault = false; // On Mac we want bw by default
+#ifdef Q_OS_MAC
+    // OEM themes are not obliged to ship mono icons
+    monoDefault = (0 == (strcmp("ownCloud", APPLICATION_NAME)));
+#endif
+    return settings.value(QLatin1String(monoIconsC), monoDefault).toBool();
 }
 
 void ConfigFile::setMonoIcons(bool useMonoIcons)
@@ -619,11 +685,11 @@ QString ConfigFile::certificatePath() const
     return retrieveData(QString(), QLatin1String(certPath)).toString();
 }
 
-void ConfigFile::setCertificatePath(const QString& cPath)
+void ConfigFile::setCertificatePath(const QString &cPath)
 {
-     QSettings settings(configFile(), QSettings::IniFormat);
-     settings.setValue( QLatin1String(certPath), cPath);
-     settings.sync();
+    QSettings settings(configFile(), QSettings::IniFormat);
+    settings.setValue(QLatin1String(certPath), cPath);
+    settings.sync();
 }
 
 QString ConfigFile::certificatePasswd() const
@@ -631,11 +697,25 @@ QString ConfigFile::certificatePasswd() const
     return retrieveData(QString(), QLatin1String(certPasswd)).toString();
 }
 
-void ConfigFile::setCertificatePasswd(const QString& cPasswd)
+void ConfigFile::setCertificatePasswd(const QString &cPasswd)
 {
-     QSettings settings(configFile(), QSettings::IniFormat);
-     settings.setValue( QLatin1String(certPasswd), cPasswd);
-     settings.sync();
+    QSettings settings(configFile(), QSettings::IniFormat);
+    settings.setValue(QLatin1String(certPasswd), cPasswd);
+    settings.sync();
+}
+
+Q_GLOBAL_STATIC(QString, g_configFileName)
+
+std::unique_ptr<QSettings> ConfigFile::settingsWithGroup(const QString &group, QObject *parent)
+{
+    if (g_configFileName()->isEmpty()) {
+        // cache file name
+        ConfigFile cfg;
+        *g_configFileName() = cfg.configFile();
+    }
+    std::unique_ptr<QSettings> settings(new QSettings(*g_configFileName(), QSettings::IniFormat, parent));
+    settings->beginGroup(group);
+    return settings;
 }
 
 }
