@@ -3,7 +3,8 @@
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; version 2 of the License.
+ * the Free Software Foundation; either version 2 of the License, or
+ * (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful, but
  * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
@@ -27,7 +28,7 @@ namespace OCC {
  * This is a job-like class to check that the server is up and that we are connected.
  * There are two entry points: checkServerAndAuth and checkAuthentication
  * checkAuthentication is the quick version that only does the propfind
- * while checkServerAndAuth is doing the 3 calls.
+ * while checkServerAndAuth is doing the 4 calls.
  *
  * We cannot use the capabilites call to test the login and the password because of
  * https://github.com/owncloud/core/issues/12930
@@ -57,10 +58,21 @@ namespace OCC {
                               |
   +---------------------------+
   |
-  +-> checkServerCapabilities (cloud/capabilities)
-        JsonApiJob
+  +-> checkServerCapabilities --------------v (in parallel)
+        JsonApiJob (cloud/capabilities)     JsonApiJob (ocs/v1.php/config)
+        |                                   +-> ocsConfigReceived
+        +-> slotCapabilitiesRecieved -+
+                                      |
+  +-----------------------------------+
+  |
+  +-> fetchUser
+        PropfindJob
         |
-        +-> slotCapabilitiesRecieved --> X
+        +-> slotUserFetched
+              AvatarJob
+              |
+              +-> slotAvatarImage --> reportResult()
+
     \endcode
  */
 class OWNCLOUDSYNC_EXPORT ConnectionValidator : public QObject
@@ -73,19 +85,20 @@ public:
         Undefined,
         Connected,
         NotConfigured,
-        ServerVersionMismatch,
-        CredentialsMissingOrWrong,
-        StatusNotFound,
-        UserCanceledCredentials,
-        ServiceUnavailable,
-        // actually also used for other errors on the authed request
-        Timeout
+        ServerVersionMismatch, // The server version is too old
+        CredentialsNotReady, // Credentials aren't ready
+        CredentialsWrong, // AuthenticationRequiredError
+        SslError, // SSL handshake error, certificate rejected by user?
+        StatusNotFound, // Error retrieving status.php
+        ServiceUnavailable, // 503 on authed request
+        MaintenanceMode, // maintenance enabled in status.php
+        Timeout // actually also used for other errors on the authed request
     };
 
-    static QString statusString( Status );
+    static QString statusString(Status);
 
     // How often should the Application ask this object to check for the connection?
-    static qint64 defaultCallingIntervalMsec() { return 32 * 1000;}
+    enum { DefaultCallingIntervalMsec = 32 * 1000 };
 
 public slots:
     /// Checks the server and the authentication.
@@ -96,29 +109,40 @@ public slots:
     void checkAuthentication();
 
 signals:
-    void connectionResult( ConnectionValidator::Status status, QStringList errors );
+    void connectionResult(ConnectionValidator::Status status, QStringList errors);
 
 protected slots:
     void slotCheckServerAndAuth();
 
-    void slotStatusFound(const QUrl&url, const QVariantMap &info);
+    void slotStatusFound(const QUrl &url, const QJsonObject &info);
     void slotNoStatusFound(QNetworkReply *reply);
-    void slotJobTimeout(const QUrl& url);
+    void slotJobTimeout(const QUrl &url);
 
     void slotAuthFailed(QNetworkReply *reply);
     void slotAuthSuccess();
 
-    void slotCapabilitiesRecieved(const QVariantMap&);
+    void slotCapabilitiesRecieved(const QJsonDocument &);
+    void slotUserFetched(const QJsonDocument &);
+#ifndef TOKEN_AUTH_ONLY
+    void slotAvatarImage(const QImage &img);
+#endif
 
 private:
     void reportResult(Status status);
     void checkServerCapabilities();
+    void fetchUser();
+    static void ocsConfigReceived(const QJsonDocument &json, AccountPtr account);
+
+    /** Sets the account's server version
+     *
+     * Returns false and reports ServerVersionMismatch for very old servers.
+     */
+    bool setAndCheckServerVersion(const QString &version);
 
     QStringList _errors;
-    AccountPtr   _account;
+    AccountPtr _account;
     bool _isCheckingServerAndAuth;
 };
-
 }
 
 #endif // CONNECTIONVALIDATOR_H
