@@ -3,7 +3,8 @@
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; version 2 of the License.
+ * the Free Software Foundation; either version 2 of the License, or
+ * (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful, but
  * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
@@ -38,15 +39,14 @@ public:
     using QNetworkCookieJar::setAllCookies;
     using QNetworkCookieJar::allCookies;
 
-    void save();
+    void save(const QString &fileName);
+    void restore(const QString &fileName);
 
 signals:
-    void newCookiesForUrl(const QList<QNetworkCookie>& cookieList, const QUrl& url);
-private:
-    void restore();
-    QList<QNetworkCookie> removeExpired(const QList<QNetworkCookie> &cookies);
-    QString storagePath() const;
+    void newCookiesForUrl(const QList<QNetworkCookie> &cookieList, const QUrl &url);
 
+private:
+    QList<QNetworkCookie> removeExpired(const QList<QNetworkCookie> &cookies);
 };
 
 } // namespace OCC
