@@ -15,21 +15,16 @@
 
 #include "propagatorjobs.h"
 #include "owncloudpropagator_p.h"
-
-#include "utility.h"
-#include "syncjournaldb.h"
-#include "syncjournalfilerecord.h"
+#include "propagateremotemove.h"
+#include "common/utility.h"
+#include "common/syncjournaldb.h"
+#include "common/syncjournalfilerecord.h"
 #include "filesystem.h"
 #include <qfile.h>
 #include <qdir.h>
 #include <qdiriterator.h>
 #include <qtemporaryfile.h>
-#if QT_VERSION < QT_VERSION_CHECK(5, 0, 0)
-#include <qabstractfileengine.h>
-#else
 #include <qsavefile.h>
-#endif
-#include <QDebug>
 #include <QDateTime>
 #include <qstack.h>
 #include <QCoreApplication>
@@ -39,6 +34,15 @@
 
 namespace OCC {
 
+Q_LOGGING_CATEGORY(lcPropagateLocalRemove, "sync.propagator.localremove", QtInfoMsg)
+Q_LOGGING_CATEGORY(lcPropagateLocalMkdir, "sync.propagator.localmkdir", QtInfoMsg)
+Q_LOGGING_CATEGORY(lcPropagateLocalRename, "sync.propagator.localrename", QtInfoMsg)
+
+QByteArray localFileIdFromFullId(const QByteArray &id)
+{
+    return id.left(8);
+}
+
 /**
  * Code inspired from Qt5's QDir::removeRecursively
  * The code will update the database in case of error.
@@ -46,39 +50,38 @@ namespace OCC {
  * in the database.  But in case of error, we need to remove the entries from the database of the files
  * that were deleted.
  *
- * \a path is relative to _propagator->_localDir + _item->_file and should start with a slash
+ * \a path is relative to propagator()->_localDir + _item->_file and should start with a slash
  */
-bool PropagateLocalRemove::removeRecursively(const QString& path)
+bool PropagateLocalRemove::removeRecursively(const QString &path)
 {
     bool success = true;
-    QString absolute = _propagator->_localDir + _item->_file + path;
+    QString absolute = propagator()->_localDir + _item->_file + path;
     QDirIterator di(absolute, QDir::AllEntries | QDir::Hidden | QDir::System | QDir::NoDotAndDotDot);
 
     QVector<QPair<QString, bool>> deleted;
 
     while (di.hasNext()) {
         di.next();
-        const QFileInfo& fi = di.fileInfo();
+        const QFileInfo &fi = di.fileInfo();
         bool ok;
         // The use of isSymLink here is okay:
         // we never want to go into this branch for .lnk files
-        bool isDir = fi.isDir() && !fi.isSymLink();
+        bool isDir = fi.isDir() && !fi.isSymLink() && !FileSystem::isJunction(fi.absoluteFilePath());
         if (isDir) {
             ok = removeRecursively(path + QLatin1Char('/') + di.fileName()); // recursive
         } else {
             QString removeError;
             ok = FileSystem::remove(di.filePath(), &removeError);
             if (!ok) {
-                _error += PropagateLocalRemove::tr("Error removing '%1': %2;").
-                    arg(QDir::toNativeSeparators(di.filePath()), removeError) + " ";
-                qDebug() << "Error removing " << di.filePath() << ':' << removeError;
+                _error += PropagateLocalRemove::tr("Error removing '%1': %2;").arg(QDir::toNativeSeparators(di.filePath()), removeError) + " ";
+                qCWarning(lcPropagateLocalRemove) << "Error removing " << di.filePath() << ':' << removeError;
             }
         }
         if (success && !ok) {
             // We need to delete the entries from the database now from the deleted vector
-            foreach(const auto &it, deleted) {
-                _propagator->_journal->deleteFileRecord(_item->_originalFile + path + QLatin1Char('/') + it.first,
-                                                        it.second);
+            foreach (const auto &it, deleted) {
+                propagator()->_journal->deleteFileRecord(_item->_originalFile + path + QLatin1Char('/') + it.first,
+                    it.second);
             }
             success = false;
             deleted.clear();
@@ -88,16 +91,17 @@ bool PropagateLocalRemove::removeRecursively(const QString& path)
         }
         if (!success && ok) {
             // This succeeded, so we need to delete it from the database now because the caller won't
-            _propagator->_journal->deleteFileRecord(_item->_originalFile + path + QLatin1Char('/') + di.fileName(),
-                                                    isDir);
+            propagator()->_journal->deleteFileRecord(_item->_originalFile + path + QLatin1Char('/') + di.fileName(),
+                isDir);
         }
     }
     if (success) {
         success = QDir().rmdir(absolute);
         if (!success) {
             _error += PropagateLocalRemove::tr("Could not remove folder '%1'")
-                .arg(QDir::toNativeSeparators(absolute)) + " ";
-            qDebug() << "Error removing folder" << absolute;
+                          .arg(QDir::toNativeSeparators(absolute))
+                + " ";
+            qCWarning(lcPropagateLocalRemove) << "Error removing folder" << absolute;
         }
     }
     return success;
@@ -105,20 +109,19 @@ bool PropagateLocalRemove::removeRecursively(const QString& path)
 
 void PropagateLocalRemove::start()
 {
-    if (_propagator->_abortRequested.fetchAndAddRelaxed(0))
+    if (propagator()->_abortRequested.fetchAndAddRelaxed(0))
         return;
 
-    QString filename = _propagator->_localDir +  _item->_file;
+    QString filename = propagator()->_localDir + _item->_file;
 
-    qDebug() << filename;
+    qCDebug(lcPropagateLocalRemove) << filename;
 
-    if( _propagator->localFileNameClash(_item->_file)) {
-        done(SyncFileItem::NormalError, tr("Could not remove %1 because of a local file name clash")
-             .arg(QDir::toNativeSeparators(filename)));
+    if (propagator()->localFileNameClash(_item->_file)) {
+        done(SyncFileItem::NormalError, tr("Could not remove %1 because of a local file name clash").arg(QDir::toNativeSeparators(filename)));
         return;
     }
 
-    if (_item->_isDirectory) {
+    if (_item->isDirectory()) {
         if (QDir(filename).exists() && !removeRecursively(QString())) {
             done(SyncFileItem::NormalError, _error);
             return;
@@ -126,23 +129,23 @@ void PropagateLocalRemove::start()
     } else {
         QString removeError;
         if (FileSystem::fileExists(filename)
-                && !FileSystem::remove(filename, &removeError)) {
+            && !FileSystem::remove(filename, &removeError)) {
             done(SyncFileItem::NormalError, removeError);
             return;
         }
     }
-    emit progress(*_item, 0);
-    _propagator->_journal->deleteFileRecord(_item->_originalFile, _item->_isDirectory);
-    _propagator->_journal->commit("Local remove");
+    propagator()->reportProgress(*_item, 0);
+    propagator()->_journal->deleteFileRecord(_item->_originalFile, _item->isDirectory());
+    propagator()->_journal->commit("Local remove");
     done(SyncFileItem::Success);
 }
 
 void PropagateLocalMkdir::start()
 {
-    if (_propagator->_abortRequested.fetchAndAddRelaxed(0))
+    if (propagator()->_abortRequested.fetchAndAddRelaxed(0))
         return;
 
-    QDir newDir(_propagator->getFilePath(_item->_file));
+    QDir newDir(propagator()->getFilePath(_item->_file));
     QString newDirStr = QDir::toNativeSeparators(newDir.path());
 
     // When turning something that used to be a file into a directory
@@ -151,22 +154,22 @@ void PropagateLocalMkdir::start()
     if (_deleteExistingFile && fi.exists() && fi.isFile()) {
         QString removeError;
         if (!FileSystem::remove(newDirStr, &removeError)) {
-            done( SyncFileItem::NormalError,
-                  tr("could not delete file %1, error: %2")
-                  .arg(newDirStr, removeError));
+            done(SyncFileItem::NormalError,
+                tr("could not delete file %1, error: %2")
+                    .arg(newDirStr, removeError));
             return;
         }
     }
 
-    if( Utility::fsCasePreserving() && _propagator->localFileNameClash(_item->_file ) ) {
-        qDebug() << "WARN: new folder to create locally already exists!";
-        done( SyncFileItem::NormalError, tr("Attention, possible case sensitivity clash with %1").arg(newDirStr) );
+    if (Utility::fsCasePreserving() && propagator()->localFileNameClash(_item->_file)) {
+        qCWarning(lcPropagateLocalMkdir) << "New folder to create locally already exists with different case:" << _item->_file;
+        done(SyncFileItem::NormalError, tr("Attention, possible case sensitivity clash with %1").arg(newDirStr));
         return;
     }
-    _propagator->addTouchedFile(newDirStr);
-    QDir localDir(_propagator->_localDir);
+    emit propagator()->touchedFile(newDirStr);
+    QDir localDir(propagator()->_localDir);
     if (!localDir.mkpath(_item->_file)) {
-        done( SyncFileItem::NormalError, tr("could not create folder %1").arg(newDirStr) );
+        done(SyncFileItem::NormalError, tr("could not create folder %1").arg(newDirStr));
         return;
     }
 
@@ -175,10 +178,13 @@ void PropagateLocalMkdir::start()
     // Adding an entry with a dummy etag to the database still makes sense here
     // so the database is aware that this folder exists even if the sync is aborted
     // before the correct etag is stored.
-    SyncJournalFileRecord record(*_item, newDirStr);
+    SyncJournalFileRecord record = _item->toSyncJournalFileRecordWithInode(newDirStr);
     record._etag = "_invalid_";
-    _propagator->_journal->setFileRecord(record);
-    _propagator->_journal->commit("localMkdir");
+    if (!propagator()->_journal->setFileRecord(record)) {
+        done(SyncFileItem::FatalError, tr("Error writing metadata to the database"));
+        return;
+    }
+    propagator()->_journal->commit("localMkdir");
 
     done(SyncFileItem::Success);
 }
@@ -190,32 +196,34 @@ void PropagateLocalMkdir::setDeleteExistingFile(bool enabled)
 
 void PropagateLocalRename::start()
 {
-    if (_propagator->_abortRequested.fetchAndAddRelaxed(0))
+    if (propagator()->_abortRequested.fetchAndAddRelaxed(0))
         return;
 
-    QString existingFile = _propagator->getFilePath(_item->_file);
-    QString targetFile = _propagator->getFilePath(_item->_renameTarget);
+    QString existingFile = propagator()->getFilePath(_item->_file);
+    QString targetFile = propagator()->getFilePath(_item->_renameTarget);
 
     // if the file is a file underneath a moved dir, the _item->file is equal
     // to _item->renameTarget and the file is not moved as a result.
     if (_item->_file != _item->_renameTarget) {
-        emit progress(*_item, 0);
-        qDebug() << "MOVE " << existingFile << " => " << targetFile;
+        propagator()->reportProgress(*_item, 0);
+        qCDebug(lcPropagateLocalRename) << "MOVE " << existingFile << " => " << targetFile;
 
         if (QString::compare(_item->_file, _item->_renameTarget, Qt::CaseInsensitive) != 0
-                && _propagator->localFileNameClash(_item->_renameTarget)) {
+            && propagator()->localFileNameClash(_item->_renameTarget)) {
             // Only use localFileNameClash for the destination if we know that the source was not
             // the one conflicting  (renaming  A.txt -> a.txt is OK)
 
             // Fixme: the file that is the reason for the clash could be named here,
             // it would have to come out the localFileNameClash function
-            done(SyncFileItem::NormalError, tr( "File %1 can not be renamed to %2 because of a local file name clash")
-                 .arg(QDir::toNativeSeparators(_item->_file)).arg(QDir::toNativeSeparators(_item->_renameTarget)) );
+            done(SyncFileItem::NormalError,
+                tr("File %1 can not be renamed to %2 because of a local file name clash")
+                    .arg(QDir::toNativeSeparators(_item->_file))
+                    .arg(QDir::toNativeSeparators(_item->_renameTarget)));
             return;
         }
 
-        _propagator->addTouchedFile(existingFile);
-        _propagator->addTouchedFile(targetFile);
+        emit propagator()->touchedFile(existingFile);
+        emit propagator()->touchedFile(targetFile);
         QString renameError;
         if (!FileSystem::rename(existingFile, targetFile, &renameError)) {
             done(SyncFileItem::NormalError, renameError);
@@ -223,25 +231,34 @@ void PropagateLocalRename::start()
         }
     }
 
-    SyncJournalFileRecord oldRecord =
-            _propagator->_journal->getFileRecord(_item->_originalFile);
-    _propagator->_journal->deleteFileRecord(_item->_originalFile);
+    SyncJournalFileRecord oldRecord;
+    propagator()->_journal->getFileRecord(_item->_originalFile, &oldRecord);
+    propagator()->_journal->deleteFileRecord(_item->_originalFile);
 
     // store the rename file name in the item.
+    const auto oldFile = _item->_file;
     _item->_file = _item->_renameTarget;
 
-    SyncJournalFileRecord record(*_item, targetFile);
-    record._path = _item->_renameTarget;
-    record._contentChecksum = oldRecord._contentChecksum;
-    record._contentChecksumType = oldRecord._contentChecksumType;
-
-    if (!_item->_isDirectory) { // Directories are saved at the end
-        _propagator->_journal->setFileRecord(record);
+    SyncJournalFileRecord record = _item->toSyncJournalFileRecordWithInode(targetFile);
+    record._path = _item->_renameTarget.toUtf8();
+    if (oldRecord.isValid()) {
+        record._checksumHeader = oldRecord._checksumHeader;
     }
-    _propagator->_journal->commit("localRename");
 
+    if (!_item->isDirectory()) { // Directories are saved at the end
+        if (!propagator()->_journal->setFileRecord(record)) {
+            done(SyncFileItem::FatalError, tr("Error writing metadata to the database"));
+            return;
+        }
+    } else {
+        if (!PropagateRemoteMove::adjustSelectiveSync(propagator()->_journal, oldFile, _item->_renameTarget)) {
+            done(SyncFileItem::FatalError, tr("Error writing metadata to the database"));
+            return;
+        }
+    }
+
+    propagator()->_journal->commit("localRename");
 
     done(SyncFileItem::Success);
 }
-
 }
