@@ -12,12 +12,10 @@
  * for more details.
  */
 
-#include <openssl/crypto.h>
-
 #include "theme.h"
-#include "version.h"
 #include "config.h"
-#include "utility.h"
+#include "common/utility.h"
+#include "version.h"
 
 #include <QtCore>
 #ifndef TOKEN_AUTH_ONLY
@@ -28,18 +26,19 @@
 #include "owncloudtheme.h"
 
 #ifdef THEME_INCLUDE
-#  define Mirall OCC // namespace hack to make old themes work
-#  define QUOTEME(M)       #M
-#  define INCLUDE_FILE(M)  QUOTEME(M)
-#  include INCLUDE_FILE(THEME_INCLUDE)
-#  undef Mirall
+#define Mirall OCC // namespace hack to make old themes work
+#define QUOTEME(M) #M
+#define INCLUDE_FILE(M) QUOTEME(M)
+#include INCLUDE_FILE(THEME_INCLUDE)
+#undef Mirall
 #endif
 
 namespace OCC {
 
-Theme* Theme::_instance = 0;
+Theme *Theme::_instance = 0;
 
-Theme* Theme::instance() {
+Theme *Theme::instance()
+{
     if (!_instance) {
         _instance = new THEME_CLASS;
         // some themes may not call the base ctor
@@ -48,13 +47,15 @@ Theme* Theme::instance() {
     return _instance;
 }
 
-Theme::~Theme() {}
+Theme::~Theme()
+{
+}
 
-QString Theme::statusHeaderText( SyncResult::Status status ) const
+QString Theme::statusHeaderText(SyncResult::Status status) const
 {
     QString resultStr;
 
-    switch( status ) {
+    switch (status) {
     case SyncResult::Undefined:
         resultStr = QCoreApplication::translate("theme", "Status undefined");
         break;
@@ -74,13 +75,13 @@ QString Theme::statusHeaderText( SyncResult::Status status ) const
         resultStr = QCoreApplication::translate("theme", "Sync Error");
         break;
     case SyncResult::SetupError:
-        resultStr = QCoreApplication::translate("theme", "Setup Error" );
+        resultStr = QCoreApplication::translate("theme", "Setup Error");
         break;
     case SyncResult::SyncPrepare:
-        resultStr = QCoreApplication::translate("theme", "Preparing to sync" );
+        resultStr = QCoreApplication::translate("theme", "Preparing to sync");
         break;
     case SyncResult::SyncAbortRequested:
-        resultStr = QCoreApplication::translate("theme", "Aborting..." );
+        resultStr = QCoreApplication::translate("theme", "Aborting...");
         break;
     case SyncResult::Paused:
         resultStr = QCoreApplication::translate("theme", "Sync is paused");
@@ -91,22 +92,22 @@ QString Theme::statusHeaderText( SyncResult::Status status ) const
 
 QString Theme::appNameGUI() const
 {
-    return QLatin1String(APPLICATION_NAME);
+    return APPLICATION_NAME;
 }
 
 QString Theme::appName() const
 {
-    return QLatin1String(APPLICATION_SHORTNAME);
+    return APPLICATION_SHORTNAME;
 }
 
 QString Theme::version() const
 {
-    return QString::fromLocal8Bit(MIRALL_VERSION_STRING);
+    return MIRALL_VERSION_STRING;
 }
 
 #ifndef TOKEN_AUTH_ONLY
 
-QIcon Theme::trayFolderIcon( const QString& backend ) const
+QIcon Theme::trayFolderIcon(const QString &backend) const
 {
     Q_UNUSED(backend)
     return applicationIcon();
@@ -116,7 +117,7 @@ QIcon Theme::trayFolderIcon( const QString& backend ) const
  * helper to load a icon from either the icon theme the desktop provides or from
  * the apps Qt resources.
  */
-QIcon Theme::themeIcon( const QString& name, bool sysTray, bool sysTrayMenuVisible ) const
+QIcon Theme::themeIcon(const QString &name, bool sysTray, bool sysTrayMenuVisible) const
 {
     QString flavor;
     if (sysTray) {
@@ -126,15 +127,15 @@ QIcon Theme::themeIcon( const QString& name, bool sysTray, bool sysTrayMenuVisib
     }
 
     QString key = name + "," + flavor;
-    QIcon & cached = _iconCache[key];
+    QIcon &cached = _iconCache[key]; // Take reference, this will also "set" the cache entry
     if (cached.isNull()) {
-        if(QIcon::hasThemeIcon(name)) {
+        if (QIcon::hasThemeIcon(name)) {
             // use from theme
             return cached = QIcon::fromTheme(name);
         }
 
         QList<int> sizes;
-        sizes <<16 << 22 << 32 << 48 << 64 << 128 << 256;
+        sizes << 16 << 22 << 32 << 48 << 64 << 128 << 256;
         foreach (int size, sizes) {
             QString pixmapName = QString::fromLatin1(":/client/theme/%1/%2-%3.png").arg(flavor).arg(name).arg(size);
             if (QFile::exists(pixmapName)) {
@@ -159,54 +160,50 @@ QIcon Theme::themeIcon( const QString& name, bool sysTray, bool sysTrayMenuVisib
         }
     }
 
-    #ifdef Q_OS_MAC
-    #if QT_VERSION >= QT_VERSION_CHECK(5, 6, 0)
+#ifdef Q_OS_MAC
+#if QT_VERSION >= QT_VERSION_CHECK(5, 6, 0)
     // This defines the icon as a template and enables automatic macOS color handling
     // See https://bugreports.qt.io/browse/QTBUG-42109
     cached.setIsMask(_mono && sysTray && !sysTrayMenuVisible);
-    #endif
-    #endif
+#endif
+#endif
 
     return cached;
 }
 
 QString Theme::hidpiFileName(const QString &fileName, QPaintDevice *dev)
 {
-    #if QT_VERSION > QT_VERSION_CHECK(5, 0, 0)
-    qreal devicePixelRatio = dev ? dev->devicePixelRatio() :
-    qApp->primaryScreen()->devicePixelRatio();
+    qreal devicePixelRatio = dev ? dev->devicePixelRatio() : qApp->primaryScreen()->devicePixelRatio();
     if (devicePixelRatio <= 1.0) {
         return fileName;
     }
     // try to find a 2x version
 
 
-
     const int dotIndex = fileName.lastIndexOf(QLatin1Char('.'));
     if (dotIndex != -1) {
         QString at2xfileName = fileName;
         at2xfileName.insert(dotIndex, QStringLiteral("@2x"));
-        if (QFile::exists(at2xfileName))  {
+        if (QFile::exists(at2xfileName)) {
             return at2xfileName;
         }
     }
-    #endif
     return fileName;
 }
 
 
 #endif
 
-Theme::Theme() :
-    QObject(0)
-    ,_mono(false)
+Theme::Theme()
+    : QObject(0)
+    , _mono(false)
 {
-
 }
 
 // If this option returns true, the client only supports one folder to sync.
 // The Add-Button is removed accordingly.
-bool Theme::singleSyncFolder() const {
+bool Theme::singleSyncFolder() const
+{
     return false;
 }
 
@@ -236,18 +233,18 @@ QString Theme::defaultClientFolder() const
     return appName();
 }
 
-QString Theme::systrayIconFlavor(bool mono, bool sysTrayMenuVisible ) const
+QString Theme::systrayIconFlavor(bool mono, bool sysTrayMenuVisible) const
 {
     Q_UNUSED(sysTrayMenuVisible)
     QString flavor;
     if (mono) {
         flavor = Utility::hasDarkSystray() ? QLatin1String("white") : QLatin1String("black");
 
-        #ifdef Q_OS_MAC
+#ifdef Q_OS_MAC
         if (sysTrayMenuVisible) {
             flavor = QLatin1String("white");
         }
-        #endif
+#endif
     } else {
         flavor = QLatin1String("colored");
     }
@@ -265,6 +262,12 @@ bool Theme::systrayUseMonoIcons() const
     return _mono;
 }
 
+bool Theme::monoIconsAvailable() const
+{
+    QString themeDir = QString::fromLatin1(":/client/theme/%1/").arg(Theme::instance()->systrayIconFlavor(true));
+    return QDir(themeDir).exists();
+}
+
 QString Theme::updateCheckUrl() const
 {
     return QLatin1String("https://updates.owncloud.com/client/");
@@ -291,20 +294,17 @@ QString Theme::gitSHA1() const
     QString devString;
 #ifdef GIT_SHA1
     const QString githubPrefix(QLatin1String(
-                                   "https://github.com/owncloud/client/commit/"));
+        "https://github.com/owncloud/client/commit/"));
     const QString gitSha1(QLatin1String(GIT_SHA1));
     devString = QCoreApplication::translate("ownCloudTheme::about()",
-                   "<p><small>Built from Git revision <a href=\"%1\">%2</a>"
-                   " on %3, %4 using Qt %5, %6</small></p>")
-            .arg(githubPrefix+gitSha1).arg(gitSha1.left(6))
-            .arg(__DATE__).arg(__TIME__)
-            .arg(QString::fromAscii(qVersion()))
-#if QT_VERSION >= QT_VERSION_CHECK(5, 0, 0)
-            .arg(QSslSocket::sslLibraryVersionString());
-#else
-            .arg(QCoreApplication::translate("ownCloudTheme::about()", "built with %1").arg(
-                     QString::fromAscii(OPENSSL_VERSION_TEXT)));
-#endif
+        "<p><small>Built from Git revision <a href=\"%1\">%2</a>"
+        " on %3, %4 using Qt %5, %6</small></p>")
+                    .arg(githubPrefix + gitSha1)
+                    .arg(gitSha1.left(6))
+                    .arg(__DATE__)
+                    .arg(__TIME__)
+                    .arg(QString::fromAscii(qVersion()))
+                    .arg(QSslSocket::sslLibraryVersionString());
 #endif
     return devString;
 }
@@ -313,27 +313,28 @@ QString Theme::about() const
 {
     QString re;
     re = tr("<p>Version %1. For more information please visit <a href='%2'>%3</a>.</p>")
-            .arg(MIRALL_VERSION_STRING).arg("http://" MIRALL_STRINGIFY(APPLICATION_DOMAIN))
-            .arg(MIRALL_STRINGIFY(APPLICATION_DOMAIN));
+             .arg(MIRALL_VERSION_STRING)
+             .arg("http://" MIRALL_STRINGIFY(APPLICATION_DOMAIN))
+             .arg(MIRALL_STRINGIFY(APPLICATION_DOMAIN));
 
     re += tr("<p>Copyright ownCloud GmbH</p>");
     re += tr("<p>Distributed by %1 and licensed under the GNU General Public License (GPL) Version 2.0.<br/>"
              "%2 and the %2 logo are registered trademarks of %1 in the "
              "United States, other countries, or both.</p>")
-            .arg(APPLICATION_VENDOR).arg(APPLICATION_NAME);
+              .arg(APPLICATION_VENDOR)
+              .arg(APPLICATION_NAME);
 
     re += gitSHA1();
     return re;
 }
 
 #ifndef TOKEN_AUTH_ONLY
-QVariant Theme::customMedia( CustomMediaType type )
+QVariant Theme::customMedia(CustomMediaType type)
 {
     QVariant re;
     QString key;
 
-    switch ( type )
-    {
+    switch (type) {
     case oCSetupTop:
         key = QLatin1String("oCSetupTop");
         break;
@@ -349,24 +350,24 @@ QVariant Theme::customMedia( CustomMediaType type )
     }
 
     QString imgPath = QString::fromLatin1(":/client/theme/colored/%1.png").arg(key);
-    if ( QFile::exists( imgPath ) ) {
-        QPixmap pix( imgPath );
-        if( pix.isNull() ) {
+    if (QFile::exists(imgPath)) {
+        QPixmap pix(imgPath);
+        if (pix.isNull()) {
             // pixmap loading hasn't succeeded. We take the text instead.
-            re.setValue( key );
+            re.setValue(key);
         } else {
-            re.setValue( pix );
+            re.setValue(pix);
         }
     }
     return re;
 }
 
-QIcon Theme::syncStateIcon( SyncResult::Status status, bool sysTray, bool sysTrayMenuVisible ) const
+QIcon Theme::syncStateIcon(SyncResult::Status status, bool sysTray, bool sysTrayMenuVisible) const
 {
     // FIXME: Mind the size!
     QString statusIcon;
 
-    switch( status ) {
+    switch (status) {
     case SyncResult::Undefined:
         // this can happen if no sync connections are configured.
         statusIcon = QLatin1String("state-information");
@@ -388,22 +389,22 @@ QIcon Theme::syncStateIcon( SyncResult::Status status, bool sysTray, bool sysTra
         break;
     case SyncResult::Error:
     case SyncResult::SetupError:
-        // FIXME: Use state-problem once we have an icon.
+    // FIXME: Use state-problem once we have an icon.
     default:
         statusIcon = QLatin1String("state-error");
     }
 
-    return themeIcon( statusIcon, sysTray, sysTrayMenuVisible );
+    return themeIcon(statusIcon, sysTray, sysTrayMenuVisible);
 }
 
-QIcon Theme::folderDisabledIcon( ) const
+QIcon Theme::folderDisabledIcon() const
 {
-    return themeIcon( QLatin1String("state-pause") );
+    return themeIcon(QLatin1String("state-pause"));
 }
 
-QIcon Theme::folderOfflineIcon(bool sysTray, bool sysTrayMenuVisible ) const
+QIcon Theme::folderOfflineIcon(bool sysTray, bool sysTrayMenuVisible) const
 {
-    return themeIcon( QLatin1String("state-offline"), sysTray, sysTrayMenuVisible );
+    return themeIcon(QLatin1String("state-offline"), sysTray, sysTrayMenuVisible);
 }
 
 QColor Theme::wizardHeaderTitleColor() const
@@ -427,7 +428,15 @@ QPixmap Theme::wizardHeaderBanner() const
     if (!c.isValid())
         return QPixmap();
 
-    QPixmap pix(QSize(750, 78));
+    QSize size(750, 78);
+    if (auto screen = qApp->primaryScreen()) {
+        // Adjust the the size if there is a different DPI. (Issue #6156)
+        // Indeed, this size need to be big enough to for the banner height, and the wizard's width
+        auto ratio = screen->logicalDotsPerInch() / 96.;
+        if (ratio > 1.)
+            size *= ratio;
+    }
+    QPixmap pix(size);
     pix.fill(wizardHeaderBackgroundColor());
     return pix;
 }
@@ -494,5 +503,30 @@ QString Theme::quotaBaseFolder() const
     return QLatin1String("/");
 }
 
+QString Theme::oauthClientId() const
+{
+    return "xdXOt13JKxym1B1QcEncf2XDkLAexMBFwiT9j6EfhhHFJhs2KM9jbjTmf8JBXE69";
+}
+
+QString Theme::oauthClientSecret() const
+{
+    return "UBntmLjC2yYCeHwsyj73Uwo9TAaecAetRwMw0xYcvNL9yRdLSUi0hUAHfvCHFeFh";
+}
+
+QString Theme::versionSwitchOutput() const
+{
+    QString helpText;
+    QTextStream stream(&helpText);
+    stream << appName()
+           << QLatin1String(" version ")
+           << version() << endl;
+#ifdef GIT_SHA1
+    stream << "Git revision " << GIT_SHA1 << endl;
+#endif
+    stream << "Using Qt " << qVersion() << ", built against Qt " << QT_VERSION_STR << endl;
+    stream << "Using '" << QSslSocket::sslLibraryVersionString() << "'" << endl;
+    return helpText;
+}
+
 
 } // end namespace client
